#pragma once

#include "ElementVisualizer.h"

#include <RobotAPI/interface/ArViz/Elements.h>

#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoMaterialBinding.h>
#include <Inventor/nodes/SoPointSet.h>

namespace armarx::viz::coin
{
    struct VisualizationPointCloud : TypedElementVisualization<SoSeparator>
    {
        using ElementType = data::ElementPointCloud;

        VisualizationPointCloud()
        {
            pclMat = new SoMaterial;

            SoMaterialBinding* pclMatBind = new SoMaterialBinding;
            pclMatBind->value = SoMaterialBinding::PER_PART;

            pclCoords = new SoCoordinate3;
            pclStyle = new SoDrawStyle;

            node->addChild(pclMat);
            node->addChild(pclMatBind);
            node->addChild(pclCoords);
            node->addChild(pclStyle);
            node->addChild(new SoPointSet);
        }

        bool update(ElementType const& element)
        {
            data::ColoredPointList const& pcl = element.points;

            int pclSize = (int)pcl.size();
            colors.resize(pclSize);
            coords.resize(pclSize);

            const float conv = 1.0f / 255.0f;
            SbColor* colorsData = colors.data();
            SbVec3f* coordsData = coords.data();
            for (int i = 0; i < pclSize; ++i)
            {
                data::ColoredPoint point = pcl[i];
                float r = point.color.r * conv;
                float g = point.color.g * conv;
                float b = point.color.b * conv;
                colorsData[i].setValue(r, g, b);
                coordsData[i].setValue(point.x, point.y, point.z);
            }
            pclMat->diffuseColor.setValuesPointer(pclSize, colors.data());
            pclMat->ambientColor.setValuesPointer(pclSize, colors.data());
            pclMat->transparency = element.transparency;

            pclCoords->point.setValuesPointer(pclSize, coords.data());

            pclStyle->pointSize = element.pointSizeInPixels;

            return true;
        }

        SoMaterial* pclMat;
        SoCoordinate3* pclCoords;
        SoDrawStyle* pclStyle;

        std::vector<SbColor> colors;
        std::vector<SbVec3f> coords;
    };
}
