#pragma once

#include <map>
#include <string>
#include <optional>

#include <SimoxUtility/caching/CacheMap.h>
#include <SimoxUtility/shapes/OrientedBox.h>

#include <ArmarXCore/core/logging/Logging.h>

#include <RobotAPI/interface/core/RobotState.h>
#include <RobotAPI/interface/objectpose/ObjectPoseObserver.h>

#include <RobotAPI/libraries/ArmarXObjects/ObjectID.h>
#include <RobotAPI/libraries/ArmarXObjects/ObjectPose.h>
#include <RobotAPI/libraries/ArmarXObjects/ObjectFinder.h>

#include "Decay.h"


namespace armarx::objpose::observer
{

    /**
     * @brief Models decay of object localizations by decreasing the confidence
     * the longer the object was not localized.
     */
    class Data : public armarx::Logging
    {
    public:

        void defineProperties(armarx::PropertyDefinitionsPtr defs, const std::string& prefix = "");

        ObjectPoseSeq getObjectPoses(IceUtil::Time now);
        ObjectPoseSeq getObjectPosesByProvider(const std::string& providerName, IceUtil::Time now);

        ObjectPose* findObjectPose(const ObjectID& objectID, const std::string& providerName = "");
        std::optional<simox::OrientedBoxf> getObjectOOBB(const ObjectID& id);

        ProviderInfo getProviderInfo(const std::string& providerName);


        AttachObjectToRobotNodeOutput attachObjectToRobotNode(const AttachObjectToRobotNodeInput& input);
        DetachObjectFromRobotNodeOutput detachObjectFromRobotNode(const DetachObjectFromRobotNodeInput& input);
        DetachAllObjectsFromRobotNodesOutput detachAllObjectsFromRobotNodes();


        void updateObjectPoses(IceUtil::Time now);
        void updateObjectPoses(
            ObjectPoseSeq& objectPoses,
            IceUtil::Time now,
            VirtualRobot::RobotPtr agent,
            bool& agentSynchronized
        ) const;
        void updateObjectPose(
            ObjectPose& objectPose,
            IceUtil::Time now,
            VirtualRobot::RobotPtr agent,
            bool& agentSynchronized
        ) const;

        /**
         * @brief If the object is attached to a robot node, update it according to the current robot state.
         *
         * If there is no attachement info in `objectPose` itself, the internal data
         * structure `attachments` is queried. If an attachment is found there,
         * it is written into the given `objectPose` (thus, it is "cached" in the
         * info `objectPose`).
         *
         * @param synchronized Indicates whether the agent is already synchronized to the current time.
         */
        void updateAttachement(ObjectPose& objectPose, VirtualRobot::RobotPtr agent,
                               bool& synchronized) const;


    public:

        RobotStateComponentInterfacePrx robotStateComponent;
        VirtualRobot::RobotPtr robot;

        ProviderInfoMap providers;

        std::map<std::string, ObjectPoseSeq> objectPoses;

        std::map<std::pair<std::string, ObjectID>, ObjectAttachmentInfo> attachments;


        ObjectFinder objectFinder;
        /// Caches results of attempts to retrieve the OOBB from ArmarXObjects.
        simox::caching::CacheMap<ObjectID, std::optional<simox::OrientedBoxf>> oobbCache;

        /// Decay model.
        Decay decay;

    };

}
