/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::ArMemDebugMemory
 * @author     Rainer Kartmann ( rainer dot kartmann at kit dot edu )
 * @date       2020
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "ArMemDebugMemory.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include <SimoxUtility/algorithm/string.h>

#include <RobotAPI/libraries/armem/core/error.h>
#include <RobotAPI/libraries/armem/server/MemoryRemoteGui.h>

namespace armarx
{
    ArMemDebugMemoryPropertyDefinitions::ArMemDebugMemoryPropertyDefinitions(std::string prefix) :
        armarx::ComponentPropertyDefinitions(prefix)
    {
    }

    ArMemDebugMemory::ArMemDebugMemory()
    {
    }

    armarx::PropertyDefinitionsPtr ArMemDebugMemory::createPropertyDefinitions()
    {
        armarx::PropertyDefinitionsPtr defs = new ArMemDebugMemoryPropertyDefinitions(getConfigIdentifier());

        defs->topic(debugObserver);
        return defs;
    }


    std::string ArMemDebugMemory::getDefaultName() const
    {
        return "ArMemDebugMemory";
    }


    void ArMemDebugMemory::onInitComponent()
    {
        memory.name() = memoryName;
    }


    void ArMemDebugMemory::onConnectComponent()
    {
        RemoteGui__createTab();
        RemoteGui_startRunningTask();
    }


    void ArMemDebugMemory::onDisconnectComponent()
    {
    }


    void ArMemDebugMemory::onExitComponent()
    {
    }



    // WRITING
    armem::data::AddSegmentsResult ArMemDebugMemory::addSegments(const armem::data::AddSegmentsInput& input, const Ice::Current&)
    {
        armem::data::AddSegmentsResult result = ComponentPluginUser::addSegments(input, addCoreSegmentOnUsage);
        tab.rebuild = true;
        return result;
    }


    armem::data::CommitResult ArMemDebugMemory::commit(const armem::data::Commit& commit, const Ice::Current&)
    {
        armem::data::CommitResult result = ComponentPluginUser::commit(commit);
        tab.rebuild = true;
        return result;
    }


    // READING

    // Inherited from Plugin



    // REMOTE GUI

    void ArMemDebugMemory::RemoteGui__createTab()
    {
        using namespace armarx::RemoteGui::Client;

        armem::server::MemoryRemoteGui mrg;
        {
            std::scoped_lock lock(memoryMutex);
            tab.memoryGroup = mrg.makeGroupBox(memory);
        }

        VBoxLayout root = {tab.memoryGroup, VSpacer()};
        RemoteGui_createTab(getName(), root, &tab);
    }


    void ArMemDebugMemory::RemoteGui_update()
    {
        if (tab.rebuild.exchange(false))
        {
            RemoteGui__createTab();
        }
    }

}
