/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::ArMemDebugMemory
 * @author     Rainer Kartmann ( rainer dot kartmann at kit dot edu )
 * @date       2020
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once


#include <ArmarXCore/core/Component.h>

#include <ArmarXCore/interface/observers/ObserverInterface.h>
#include <ArmarXGui/libraries/ArmarXGuiComponentPlugins/LightweightRemoteGuiComponentPlugin.h>
#include <RobotAPI/libraries/RobotAPIComponentPlugins/ArVizComponentPlugin.h>

#include <RobotAPI/libraries/armem/server/ComponentPlugin.h>


namespace armarx
{
    /**
     * @class ArMemDebugMemoryPropertyDefinitions
     * @brief Property definitions of `ArMemDebugMemory`.
     */
    class ArMemDebugMemoryPropertyDefinitions :
        public armarx::ComponentPropertyDefinitions
    {
    public:
        ArMemDebugMemoryPropertyDefinitions(std::string prefix);
    };


    /**
     * @defgroup Component-ArMemDebugMemory ArMemDebugMemory
     * @ingroup RobotAPI-Components
     * A description of the component ArMemDebugMemory.
     *
     * @class ArMemDebugMemory
     * @ingroup Component-ArMemDebugMemory
     * @brief Brief description of class ArMemDebugMemory.
     *
     * Detailed description of class ArMemDebugMemory.
     */
    class ArMemDebugMemory :
        virtual public armarx::Component
        , virtual public armem::server::ComponentPluginUser
        , virtual public LightweightRemoteGuiComponentPluginUser
    // , virtual public armarx::ArVizComponentPluginUser
    {
    public:
        ArMemDebugMemory();

        /// @see armarx::ManagedIceObject::getDefaultName()
        std::string getDefaultName() const override;

        // WritingInterface interface
    public:
        armem::data::AddSegmentsResult addSegments(const armem::data::AddSegmentsInput& input, const Ice::Current&) override;
        armem::data::CommitResult commit(const armem::data::Commit& commit, const Ice::Current&) override;

        // LightweightRemoteGuiComponentPluginUser interface
    public:
        void RemoteGui__createTab();
        void RemoteGui_update() override;


    protected:
        /// @see armarx::ManagedIceObject::onInitComponent()
        void onInitComponent() override;

        /// @see armarx::ManagedIceObject::onConnectComponent()
        void onConnectComponent() override;

        /// @see armarx::ManagedIceObject::onDisconnectComponent()
        void onDisconnectComponent() override;

        /// @see armarx::ManagedIceObject::onExitComponent()
        void onExitComponent() override;

        /// @see PropertyUser::createPropertyDefinitions()
        armarx::PropertyDefinitionsPtr createPropertyDefinitions() override;


    private:
        /// Debug observer. Used to visualize e.g. time series.
        armarx::DebugObserverInterfacePrx debugObserver;

        std::string memoryName = "DebugMemory";
        bool addCoreSegmentOnUsage = true;

        struct RemoteGuiTab : RemoteGui::Client::Tab
        {
            std::atomic_bool rebuild = false;
            RemoteGui::Client::GroupBox memoryGroup;
        };
        RemoteGuiTab tab;

    };
}
