#pragma once


// STD/STL
#include <functional>
#include <unordered_map>
#include <vector>

// RobotAPI
#include <RobotAPI/interface/armem/server/ReadingMemoryInterface.h>
#include <RobotAPI/interface/armem/client/MemoryListenerInterface.h>
#include <RobotAPI/libraries/armem/core/ice_conversions.h>
#include <RobotAPI/libraries/armem/core/Memory.h>

#include "Query.h"


namespace armarx::armem::client
{

    /**
     * @brief Reads data from a memory server.
     */
    class Reader
    {

        using callback = std::function<void(const MemoryID& subscriptionID, const std::vector<MemoryID>& updatedSnapshotIDs)>;
        template <class CalleeT>
        using member_callback = void(CalleeT::*)(const MemoryID& subscriptionID, const std::vector<MemoryID>& updatedSnapshotIDs);

    public:

        /**
         * @brief Construct a memory reader.
         * @param memory The memory proxy.
         */
        Reader(server::ReadingMemoryInterfacePrx memory = nullptr);

        void setReadingMemory(server::ReadingMemoryInterfacePrx memory);


        QueryResult query(const QueryInput& input);
        armem::query::data::Result query(const armem::query::data::Input& input);

        QueryResult query(armem::query::data::MemoryQueryPtr query, DataMode dataMode = DataMode::WithData);
        QueryResult query(const armem::query::data::MemoryQuerySeq& queries, DataMode dataMode = DataMode::WithData);


        QueryResult getAll(DataMode dataMode = DataMode::WithData);
        QueryResult getLatestSnapshots(DataMode dataMode = DataMode::WithData);


        void subscribe(const MemoryID& subscriptionID, callback callback);
        /**
         * Subscribe with a class member function:
         * @code
         * reader.subscribe(entityID, this, &This::myCallback);
         * @endcode
         */
        template <class CalleeT>
        void subscribe(const MemoryID& subscriptionID, CalleeT* callee, member_callback<CalleeT> callback)
        {
            auto cb = [callee, callback](const MemoryID & subscriptionID, const std::vector<MemoryID>& updatedSnapshotIDs)
            {
                (callee->*callback)(subscriptionID, updatedSnapshotIDs);
            };
            subscribe(subscriptionID, cb);
        }
        /// Function handling updates from the MemoryListener ice topic.
        void updated(const std::vector<MemoryID>& updatedIDs);


        inline operator bool() const
        {
            return bool(memory);
        }

    public:

        server::ReadingMemoryInterfacePrx memory;

    private:

        std::unordered_map<MemoryID, callback> callbacks;

    };

}
