#pragma once

#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/aroncore/navigators/datanavigator/AronDictDataNavigator.h>

#include "../core/Time.h"

#include "Commit.h"
#include "detail/MemoryItem.h"


namespace armarx::armem
{

    /**
     * @brief Metadata of an entity instance.
     */
    struct EntityInstanceMetadata
    {
        /// Time when this value was created.
        Time timeCreated;
        /// Time when this value was sent to the memory.
        Time timeSent;
        /// Time when this value has arrived at the memory.
        Time timeArrived;

        /// An optional confidence, may be used for things like decay.
        float confidence = 1.0;


        bool operator==(const EntityInstanceMetadata& other) const;
        inline bool operator!=(const EntityInstanceMetadata& other) const
        {
            return !(*this == other);
        }
    };


    /**
     * @brief Data of a single entity instance.
     */
    class EntityInstance : public detail::MemoryItem
    {
    public:

        EntityInstance();
        EntityInstance(int index, const MemoryID& parentID = {});
        EntityInstance(const MemoryID& id);


        inline int& index()
        {
            return id().instanceIndex;
        }
        inline int index() const
        {
            return id().instanceIndex;
        }

        EntityInstanceMetadata& metadata()
        {
            return _metadata;
        }
        inline const EntityInstanceMetadata& metadata() const
        {
            return _metadata;
        }

        inline armarx::aron::datanavigator::AronDictDataNavigatorPtr data() const
        {
            return _data;
        }
        void setData(const aron::datanavigator::AronDictDataNavigatorPtr& data)
        {
            this->_data = data;
        }


        /**
         * @brief Fill `*this` with the update's values.
         * @param update The update.
         * @param index The instances index.
         */
        void update(const EntityUpdate& update, int index);


        EntityInstance copy(bool withData = true) const;

        bool equalsDeep(const EntityInstance& other) const;


        // MemoryItem interface
    public:
        std::string getKeyString() const override;
        std::string getLevelName() const override;


    private:

        /// The metadata.
        EntityInstanceMetadata _metadata;

        /// The data. May be nullptr.
        armarx::aron::datanavigator::AronDictDataNavigatorPtr _data;

    };

}
