#pragma once

#include "EntityContainer.h"

#include <RobotAPI/libraries/aron/aroncore/navigators/typenavigator/AronObjectTypeNavigator.h>

namespace armarx::armem::detail
{

    /**
     * @brief An entity container with a specific (Aron) type.
     */
    template <class _ValueT, class Derived>
    class TypedEntityContainer : public EntityContainer<_ValueT, Derived>
    {
        using ValueT = _ValueT;
        // using Base = EntityContainer<ValueT, Derived>;

    public:

        TypedEntityContainer(aron::typenavigator::AronObjectTypeNavigatorPtr aronType = nullptr) : _aronType(aronType)
        {}
        TypedEntityContainer(const MemoryID& id, aron::typenavigator::AronObjectTypeNavigatorPtr aronType = nullptr) :
            EntityContainer<ValueT, Derived>(id), _aronType(aronType)
        {}

        virtual ~TypedEntityContainer() override
        {}


        bool hasAronType() const
        {
            return _aronType != nullptr;
        }
        aron::typenavigator::AronObjectTypeNavigatorPtr& aronType()
        {
            return _aronType;
        }
        aron::typenavigator::AronObjectTypeNavigatorPtr aronType() const
        {
            return _aronType;
        }



        // MemoryContainer interface
    protected:
        virtual void _copySelf(Derived& other) const override
        {
            EntityContainer<ValueT, Derived>::_copySelf(other);
            other._aronType = this->_aronType;
        }


    private:

        /// The expected Aron type. May be nullptr, in which case no type information is available.
        aron::typenavigator::AronObjectTypeNavigatorPtr _aronType;

    };


}
