#include "ice_conversions.h"


namespace armarx
{

    /*
        void armem::fromIce(long ice, Time& time)
        {
            time = Time::microSeconds(ice);
        }

        void armem::toIce(long& ice, const Time& time)
        {
            ice = time.toMicroSeconds();
        }
    */

    void armem::toIce(data::MemoryID& ice, const MemoryID& id)
    {
        ice.memoryName = id.memoryName;
        ice.coreSegmentName = id.coreSegmentName;
        ice.providerSegmentName = id.providerSegmentName;
        ice.entityName = id.entityName;
        toIce(ice.timestampMicroSeconds, id.timestamp);
        ice.instanceIndex = id.instanceIndex;
    }

    void armem::fromIce(const data::MemoryID& ice, MemoryID& id)
    {
        id.memoryName = ice.memoryName;
        id.coreSegmentName = ice.coreSegmentName;
        id.providerSegmentName = ice.providerSegmentName;
        id.entityName = ice.entityName;
        fromIce(ice.timestampMicroSeconds, id.timestamp);
        id.instanceIndex = ice.instanceIndex;
    }


    void armem::fromIce(const data::Commit& ice, Commit& commit)
    {
        commit.updates.clear();
        for (const auto& ice_update : ice.updates)
        {
            EntityUpdate& update = commit.updates.emplace_back();
            fromIce(ice_update, update);
        }
    }

    void armem::toIce(data::Commit& ice, const Commit& commit)
    {
        ice.updates.clear();
        for (const auto& update : commit.updates)
        {
            data::EntityUpdate& ice_update = ice.updates.emplace_back();
            toIce(ice_update, update);
        }
    }


    void armem::fromIce(const data::CommitResult& ice, CommitResult& result)
    {
        result.results.clear();
        for (const auto& ice_res : ice.results)
        {
            EntityUpdateResult& res = result.results.emplace_back();
            fromIce(ice_res, res);
        }
    }

    void armem::toIce(data::CommitResult& ice, const CommitResult& result)
    {
        ice.results.clear();
        for (const auto& res : result.results)
        {
            data::EntityUpdateResult& ice_res = ice.results.emplace_back();
            toIce(ice_res, res);
        }
    }


    void armem::fromIce(const data::EntityUpdate& ice, EntityUpdate& update)
    {
        fromIce(ice.entityID, update.entityID);

        update.instancesData.clear();
        update.instancesData.reserve(ice.instancesData.size());
        std::transform(ice.instancesData.begin(), ice.instancesData.end(), std::back_inserter(update.instancesData),
                       aron::datanavigator::AronDictDataNavigator::FromAronDictPtr);

        update.timeCreated = Time::microSeconds(ice.timeCreatedMicroSeconds);

        update.confidence = ice.confidence;
        update.timeSent = Time::microSeconds(ice.timeSentMicroSeconds);
    }

    void armem::toIce(data::EntityUpdate& ice, const EntityUpdate& update)
    {
        toIce(ice.entityID, update.entityID);

        ice.instancesData.clear();
        ice.instancesData.reserve(update.instancesData.size());
        std::transform(update.instancesData.begin(), update.instancesData.end(), std::back_inserter(ice.instancesData),
                       aron::datanavigator::AronDictDataNavigator::ToAronDictPtr);

        ice.timeCreatedMicroSeconds = update.timeCreated.toMicroSeconds();

        ice.confidence = update.confidence;
        ice.timeSentMicroSeconds = update.timeSent.toMicroSeconds();
    }


    void armem::fromIce(const data::EntityUpdateResult& ice, EntityUpdateResult& result)
    {
        result.success = ice.success;
        fromIce(ice.snapshotID, result.snapshotID);
        result.timeArrived = Time::microSeconds(ice.timeArrivedMicroSeconds);
        result.errorMessage = ice.errorMessage;
    }

    void armem::toIce(data::EntityUpdateResult& ice, const EntityUpdateResult& result)
    {
        ice.success = result.success;
        toIce(ice.snapshotID, result.snapshotID);
        ice.timeArrivedMicroSeconds = result.timeArrived.toMicroSeconds();
        ice.errorMessage = result.errorMessage;
    }


    void armem::fromIce(const data::Commit& ice, Commit& commit, Time timeArrived)
    {
        commit.updates.clear();
        for (const auto& ice_update : ice.updates)
        {
            EntityUpdate& update = commit.updates.emplace_back(EntityUpdate());
            fromIce(ice_update, update, timeArrived);
        }
    }


    void armem::fromIce(const data::EntityUpdate& ice, EntityUpdate& update, Time timeArrived)
    {
        armem::fromIce(ice, update);
        update.timeArrived = timeArrived;
    }


    void armem::detail::toIceItem(data::detail::MemoryItem& ice, const MemoryItem& item)
    {
        toIce(ice.id, item.id());
    }

    void armem::detail::fromIceItem(const data::detail::MemoryItem& ice, MemoryItem& item)
    {
        fromIce(ice.id, item.id());
    }


    void armem::toIce(data::EntityInstanceMetadata& ice, const EntityInstanceMetadata& metadata)
    {
        ice.confidence = metadata.confidence;
        toIce(ice.timeArrivedMicroSeconds, metadata.timeArrived);
        toIce(ice.timeCreatedMicroSeconds, metadata.timeCreated);
        toIce(ice.timeSentMicroSeconds, metadata.timeSent);
    }
    void armem::fromIce(const data::EntityInstanceMetadata& ice, EntityInstanceMetadata& metadata)
    {
        metadata.confidence = ice.confidence;
        fromIce(ice.timeArrivedMicroSeconds, metadata.timeArrived);
        fromIce(ice.timeCreatedMicroSeconds, metadata.timeCreated);
        fromIce(ice.timeSentMicroSeconds, metadata.timeSent);
    }

    void armem::toIce(data::EntityInstance& ice, const EntityInstance& data)
    {
        detail::toIceItem(ice, data);

        if (data.data())
        {
            ice.data = data.data()->toAronDictPtr();
        }
        toIce(ice.metadata, data.metadata());
    }
    void armem::fromIce(const data::EntityInstance& ice, EntityInstance& data)
    {
        detail::fromIceItem(ice, data);

        if (ice.data)
        {
            data.setData(aron::datanavigator::AronDictDataNavigator::FromAronDictPtr(ice.data));
        }
        fromIce(ice.metadata, data.metadata());
    }


    void armem::toIce(data::EntitySnapshot& ice, const EntitySnapshot& snapshot)
    {
        detail::toIceItem(ice, snapshot);

        toIce(ice.instances, snapshot.instances);
    }
    void armem::fromIce(const data::EntitySnapshot& ice, EntitySnapshot& snapshot)
    {
        detail::fromIceItem(ice, snapshot);

        fromIce(ice.instances, snapshot.instances);
    }

    void armem::toIce(data::Entity& ice, const Entity& entity)
    {
        detail::toIceItem(ice, entity);

        toIce(ice.history, entity.history);
    }
    void armem::fromIce(const data::Entity& ice, Entity& entity)
    {
        detail::fromIceItem(ice, entity);

        fromIce(ice.history, entity.history);
    }


    void armem::toIce(data::ProviderSegment& ice, const ProviderSegment& providerSegment)
    {
        detail::toIceItem(ice, providerSegment);

        if (providerSegment.hasAronType())
        {
            ice.aronType = providerSegment.aronType()->getResult();
        }
        toIce(ice.entities, providerSegment.entities);
    }
    void armem::fromIce(const data::ProviderSegment& ice, ProviderSegment& providerSegment)
    {
        detail::fromIceItem(ice, providerSegment);

        if (ice.aronType)
        {
            providerSegment.aronType() = aron::typenavigator::AronObjectTypeNavigator::DynamicCastAndCheck(
                                             aron::typenavigator::AronTypeNavigator::FromAronType(ice.aronType)
                                         );
        }
        fromIce(ice.entities, providerSegment.entities);
    }

    void armem::toIce(data::CoreSegment& ice, const CoreSegment& coreSegment)
    {
        detail::toIceItem(ice, coreSegment);

        if (coreSegment.hasAronType())
        {
            ice.aronType = coreSegment.aronType()->getResult();
        }
        toIce(ice.providerSegments, coreSegment.providerSegments);
    }
    void armem::fromIce(const data::CoreSegment& ice, CoreSegment& coreSegment)
    {
        detail::fromIceItem(ice, coreSegment);

        if (ice.aronType)
        {
            coreSegment.aronType() = aron::typenavigator::AronObjectTypeNavigator::DynamicCastAndCheck(
                                         aron::typenavigator::AronTypeNavigator::FromAronType(ice.aronType)
                                     );
        }
        fromIce(ice.providerSegments, coreSegment.providerSegments);
    }

    void armem::toIce(data::Memory& ice, const Memory& memory)
    {
        detail::toIceItem(ice, memory);

        toIce(ice.coreSegments, memory.coreSegments);
    }
    void armem::fromIce(const data::Memory& ice, Memory& memory)
    {
        detail::fromIceItem(ice, memory);

        fromIce(ice.coreSegments, memory.coreSegments);
    }

}
