/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <string>
#include <filesystem>

// ArmarX
#include <RobotAPI/interface/aron.h>

#include <RobotAPI/libraries/armem/core/Time.h>


namespace armarx::armem::io
{
    class DiskReaderWriter
    {
    public:
        DiskReaderWriter(bool createIfNotExistent);
        DiskReaderWriter(const std::filesystem::path& rootPath, bool createIfNotExistent);

        virtual ~DiskReaderWriter();


    protected:
        virtual std::string getEntityInstanceSuffix() const = 0;
        virtual std::string getTypeSuffix() const = 0;

    protected:
        std::filesystem::path rootPath;

        const std::string DISK_READER_WRITER_DATA_FIELD            = "DATA";
        const std::string DISK_READER_WRITER_TIME_STORED_FIELD     = "STORAGE_MANAGER_METADATA__TIME_STORED";
        const std::string DISK_READER_WRITER_TIME_CREATED_FIELD    = "ENTITY_METADATA__TIME_CREATED";
        const std::string DISK_READER_WRITER_TIME_SENT_FIELD       = "ENTITY_METADATA__TIME_SENT";
        const std::string DISK_READER_WRITER_TIME_ARRIVED_FIELD    = "ENTITY_METADATA__TIME_ARRIVED";
        const std::string DISK_READER_WRITER_CONFIDENCE_FIELD      = "ENTITY_METADATA__CONFIDENCE";

    };

    using DiskReaderWriterPtr = std::shared_ptr<DiskReaderWriter>;
}
