/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// Base Class
#include "../DiskWriter.h"


namespace armarx::armem::io
{
    class NlohmannJSONDiskWriter;
    using NlohmannJSONDiskWriterPtr = std::shared_ptr<NlohmannJSONDiskWriter>;

    class NlohmannJSONDiskWriter : virtual public DiskWriter
    {
    public:
        NlohmannJSONDiskWriter(bool createFolder);
        NlohmannJSONDiskWriter(const std::string& rootPath, bool createFolder);


    protected:
        std::string getDataAsString(const aron::datanavigator::AronDictDataNavigatorPtr&) const override;
        std::string getTypeAsString(const aron::typenavigator::AronObjectTypeNavigatorPtr&) const override;

        std::string getEntityInstanceSuffix() const override
        {
            return ".data.json";
        }

        std::string getTypeSuffix() const override
        {
            return ".type.json";
        }
    };
}
