#pragma once

#include <sstream>
#include <stack>

#include <ArmarXGui/libraries/RemoteGui/Client/Widgets.h>

#include <RobotAPI/libraries/aron/aroncore/navigators/visitors/DataVisitor.h>


namespace armarx::armem::server
{

    struct RemoteGuiAronDataVisitor : public aron::visitor::DataVisitor
    {
        using GroupBox = armarx::RemoteGui::Client::GroupBox;
        using GridLayout = armarx::RemoteGui::Client::GridLayout;
        using Label = armarx::RemoteGui::Client::Label;


        struct Group
        {
            Group(const std::string& label = {});

            GroupBox groupBox;
            GridLayout layout;
            int nextRow = 0;
        };

        std::stack<Group> groups;
        GroupBox result;


        virtual ~RemoteGuiAronDataVisitor() override;


        bool visitEnter(const std::string& key, DictDataNavigator& n) override
        {
            return visitEnter(key, "dict", n.childrenSize());
        }
        bool visitExit(DictDataNavigator&) override
        {
            return visitExit();
        }

        bool visitEnter(const std::string& key, ListDataNavigator& n) override
        {
            return visitEnter(key, "list", n.childrenSize());
        }
        bool visitExit(ListDataNavigator&) override
        {
            return visitExit();
        }

        // Same for Dict and List
        bool visitEnter(const std::string& key, const std::string& type, size_t size);
        bool visitExit();

        // Do not hide base overloads.
        using DataVisitor::visitEnter;
        using DataVisitor::visitExit;


        bool visit(const std::string& key, BoolDataNavigator& b) override
        {
            this->addValueLabel(key, b, "bool");
            return true;
        }
        bool visit(const std::string& key, DoubleDataNavigator& d) override
        {
            this->addValueLabel(key, d, "double");
            return true;
        }
        bool visit(const std::string& key, FloatDataNavigator& f) override
        {
            this->addValueLabel(key, f, "float");
            return true;
        }
        bool visit(const std::string& key, IntDataNavigator& i) override
        {
            this->addValueLabel(key, i, "int");
            return true;
        }
        bool visit(const std::string& key, LongDataNavigator& l) override
        {
            this->addValueLabel(key, l, "long");
            return true;
        }
        bool visit(const std::string& key, StringDataNavigator& string) override
        {
            this->addValueLabel(key, string, "string");
            return true;
        }

        bool visit(const std::string& key, NDArrayDataNavigator& array) override
        {
            this->addValueLabel(key, array, "ND Array");
            return true;
        }


    private:

        template <class Navigator>
        void addValueLabel(const std::string& key, Navigator& n, const std::string& typeName)
        {
            checkGroupsNotEmpty();
            Group& g = groups.top();
            g.layout
            .add(Label(key), {g.nextRow, 0})
            .add(Label("(" + typeName + ")"), {g.nextRow, 1})
            .add(Label("= " + getValueText(n)), {g.nextRow, 2})
            ;
            ++g.nextRow;
        }

        template <class Navigator>
        std::string getValueText(Navigator& n)
        {
            std::stringstream ss;
            streamValueText(n, ss);
            return ss.str();
        }

        template <class Navigator>
        void streamValueText(Navigator& n, std::stringstream& ss)
        {
            ss << n.getValue();
        }
        void streamValueText(StringDataNavigator& n, std::stringstream& ss);
        void streamValueText(NDArrayDataNavigator& n, std::stringstream& ss);

        void checkGroupsNotEmpty() const;

    };


}
