#pragma once

#include <RobotAPI/interface/armem/query.h>

#include <RobotAPI/libraries/armem/core/DataMode.h>


namespace armarx::armem::query_proc
{

    /**
     * @brief Base class for memory query processors.
     */
    template <class DataT, class QueryT>
    class BaseQueryProcessor
    {
    public:

        using QueryPtrT = ::IceInternal::Handle<QueryT>;
        using QuerySeqT = std::vector<QueryPtrT>;


    public:

        BaseQueryProcessor(DataMode dataMode = DataMode::WithData) :
            dataMode(dataMode)
        {}

        virtual ~BaseQueryProcessor()
        {}


        DataT process(const QueryT& query, const DataT& data) const
        {
            DataT result = data.copyEmpty();
            this->process(result, query, data);
            return result;
        }
        DataT process(const QueryPtrT& query, const DataT& data) const
        {
            return this->process(*query, *data);
        }

        DataT process(const QuerySeqT& queries, const DataT& data) const
        {
            DataT result = data.copyEmpty();
            this->process(result, queries, data);
            return result;
        }


        void process(DataT& result, const QuerySeqT& queries, const DataT& data) const
        {
            for (const auto& query : queries)
            {
                this->process(result, *query, data);
            }
        }

        virtual void process(DataT& result, const QueryT& query, const DataT& data) const = 0;


    protected:

        DataMode dataMode;

    };

}
