#pragma once

#include <RobotAPI/interface/armem/query.h>

#include <RobotAPI/libraries/armem/core/Memory.h>

#include "BaseQueryProcessor.h"


namespace armarx::armem::query_proc
{

    /**
     * @brief Handles memory queries.
     */
    class EntityQueryProcessor : public BaseQueryProcessor<Entity, armem::query::data::EntityQuery>
    {
    public:

        EntityQueryProcessor(DataMode dataMode = DataMode::WithData) :
            BaseQueryProcessor(dataMode)
        {}


        using BaseQueryProcessor::process;

        void process(Entity& result, const armem::query::data::EntityQuery& query, const Entity& entity) const override;

        void process(Entity& result, const armem::query::data::entity::All& query, const Entity& entity) const;
        void process(Entity& result, const armem::query::data::entity::Single& query, const Entity& entity) const;
        void process(Entity& result, const armem::query::data::entity::TimeRange& query, const Entity& entity) const;
        void process(Entity& result, const armem::query::data::entity::IndexRange& query, const Entity& entity) const;

        void process(Entity& result, const Time& min, const Time& max, const Entity& entity,
                     const armem::query::data::EntityQuery& query) const;


        data::Entity processToIce(const armem::query::data::EntityQuery& query, const Entity& entity) const;


        static size_t negativeIndexSemantics(long index, size_t size);

    private:

        void addResultSnapshot(Entity& result, Entity::ContainerT::const_iterator it) const;
        void addResultSnapshot(Entity& result, const EntitySnapshot& snapshot) const;

    };

}
