#pragma once

#include <sstream>
#include <stack>

#include <QTreeWidget>
#include <QLabel>

#include <RobotAPI/libraries/aron/aroncore/navigators/visitors/TypedDataVisitor.h>
#include "InstanceTreeDataVisitorBase.h"


namespace armarx::armem::gui
{

    class InstanceTreeTypedDataVisitor :
        public aron::visitor::TypedDataVisitor,
        public InstanceTreeDataVisitorBase
    {
    public:

        using InstanceTreeDataVisitorBase::InstanceTreeDataVisitorBase;


        bool visitEnter(DictTypeNavigator& type, const std::string& key, DictDataNavigator& data) override
        {
            return _visitEnter(key, sanitizeTypeName(type.getName()), data.childrenSize());
        }
        bool visitExit(DictTypeNavigator&, DictDataNavigator&) override
        {
            return _visitExit();
        }

        bool visitEnter(ObjectTypeNavigator& type, const std::string& key, DictDataNavigator& data) override
        {
            return _visitEnter(key, sanitizeTypeName(type.getName()), data.childrenSize());
        }
        bool visitExit(ObjectTypeNavigator&, DictDataNavigator&) override
        {
            return _visitExit();
        }

        bool visitEnter(ListTypeNavigator& type, const std::string& key, ListDataNavigator& data) override
        {
            return _visitEnter(key, sanitizeTypeName(type.getName()), data.childrenSize());
        }
        bool visitExit(ListTypeNavigator&, ListDataNavigator&) override
        {
            return _visitExit();
        }

        bool visitEnter(TupleTypeNavigator& type, const std::string& key, ListDataNavigator& data) override
        {
            return _visitEnter(key, sanitizeTypeName(type.getName()), data.childrenSize());
        }
        bool visitExit(TupleTypeNavigator&, ListDataNavigator&) override
        {
            return _visitExit();
        }


        bool visit(BoolTypeNavigator& type, const std::string& key, BoolDataNavigator& data) override
        {
            return this->addValueRow(key, data, type);
        }
        bool visit(DoubleTypeNavigator& type, const std::string& key, DoubleDataNavigator& data) override
        {
            return this->addValueRow(key, data, type);
        }
        bool visit(FloatTypeNavigator& type, const std::string& key, FloatDataNavigator& data) override
        {
            return this->addValueRow(key, data, type);
        }
        bool visit(IntTypeNavigator& type, const std::string& key, IntDataNavigator& data) override
        {
            return this->addValueRow(key, data, type);
        }
        bool visit(LongTypeNavigator& type, const std::string& key, LongDataNavigator& data) override
        {
            return this->addValueRow(key, data, type);
        }
        bool visit(StringTypeNavigator& type, const std::string& key, StringDataNavigator& data) override
        {
            return this->addValueRow(key, data, type);
        }


        bool visit(EigenMatrixTypeNavigator& type, const std::string& key, NDArrayDataNavigator& data) override
        {
            return this->addValueRow(key, data, type);
        }
        bool visit(IVTCByteImageTypeNavigator& type, const std::string& key, NDArrayDataNavigator& data) override
        {
            return this->addValueRow(key, data, type);
        }
        bool visit(OpenCVMatTypeNavigator& type, const std::string& key, NDArrayDataNavigator& data) override
        {
            return this->addValueRow(key, data, type);
        }
        bool visit(PCLPointCloudTypeNavigator& type, const std::string& key, NDArrayDataNavigator& data) override
        {
            return this->addValueRow(key, data, type);
        }


        std::string sanitizeTypeName(const std::string& typeName) const;


    protected:

        template <class DataNavigatorT>
        bool addValueRow(const std::string& key, DataNavigatorT& dataNavigator, const TypeNavigator& type)
        {
            if (items.size() > 0)
            {
                QTreeWidgetItem* item = new QTreeWidgetItem(this->makeValueRowStrings(key, dataNavigator, sanitizeTypeName(type.getName())));
                items.top()->addChild(item);

                item->setData(columnKey, Qt::UserRole, serializeAronPath(dataNavigator.getPath()));
                item->setData(columnType, Qt::UserRole, int(type.getDescriptor()));
            }
            return true;
        }

    public:

        static QStringList serializeAronPath(const aron::AronPath& path);
        static aron::AronPath deserializeAronPath(const QStringList& qpath);


    };

}

