/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "AronObjectTypeCppSerializer.h"


namespace armarx
{
    namespace aron
    {
        namespace codegeneration
        {
            namespace classwriter
            {
                namespace cppSerializer
                {

                    const std::string AronObjectTypeCppSerializer::EXTENDS_ITERATOR_ACCESSOR = "_extends";

                    // constructors
                    AronObjectTypeCppSerializer::AronObjectTypeCppSerializer(const typenavigator::AronObjectTypeNavigatorPtr& e) :
                        AronTypeCppSerializer(e->getObjectName(), simox::meta::get_type_name(typeid(data::AronDict)), simox::meta::get_type_name(typeid(type::AronObjectType))),
                        typenavigator(e)
                    {
                        //AddObjectTypeToAllObjectTypesList(AronObjectTypeNavigatorPtr(this));
                    }

                    std::vector<CppFieldPtr> AronObjectTypeCppSerializer::getPublicVariableDeclarations(const std::string& name) const
                    {
                        if (name == "") // object defintiion
                        {
                            std::vector<CppFieldPtr> fields;
                            for (const auto& [key, member] : typenavigator->getAcceptedTypes())
                            {
                                auto member_s = FromAronTypeNaviagtorPtr(member);
                                std::vector<CppFieldPtr> member_fields = member_s->getPublicVariableDeclarations(key);
                                fields.insert(fields.end(), member_fields.begin(), member_fields.end());
                            }
                            return fields;
                        }
                        else // object usage
                        {
                            CppFieldPtr field = CppFieldPtr(new CppField(getCppTypename(), name));
                            return {field};
                        };
                    }

                    std::vector<std::pair<std::string, std::string>> AronObjectTypeCppSerializer::getCtorInitializers(const std::string&) const
                    {
                        return {};
                    }

                    CppBlockPtr AronObjectTypeCppSerializer::getCtorBlock(const std::string&) const
                    {
                        CppBlockPtr b = CppBlockPtr(new CppBlock());
                        return b;
                    }

                    CppBlockPtr AronObjectTypeCppSerializer::getResetBlock(const std::string& accessor) const
                    {
                        CppBlockPtr b = CppBlockPtr(new CppBlock());
                        if (accessor == "")
                        {
                            if (typenavigator->getExtends() != nullptr)
                            {
                                const auto extends_s = FromAronTypeNaviagtorPtr(typenavigator->getExtends());
                                b->addLine(extends_s->getCppTypename() + "::reset();");
                            }

                            for (const auto& [key, child] : typenavigator->getAcceptedTypes())
                            {
                                auto child_s = FromAronTypeNaviagtorPtr(child);
                                CppBlockPtr b2 = child_s->getResetBlock((accessor != "" ? (accessor + "." + key) : key));
                                b->appendBlock(b2);
                            }
                        }
                        else
                        {
                            b->addLine(accessor + ".reset();");
                        }
                        return b;
                    }

                    CppBlockPtr AronObjectTypeCppSerializer::getInitializeBlock(const std::string& accessor) const
                    {
                        CppBlockPtr b = CppBlockPtr(new CppBlock());

                        if (accessor == "")
                        {
                            if (typenavigator->getExtends() != nullptr)
                            {
                                const auto extends_s = FromAronTypeNaviagtorPtr(typenavigator->getExtends());
                                b->addLine(extends_s->getCppTypename() + "::initialize();");
                            }

                            for (const auto& [key, child] : typenavigator->getAcceptedTypes())
                            {
                                const auto child_s = FromAronTypeNaviagtorPtr(child);
                                CppBlockPtr b2 = child_s->getInitializeBlock((accessor != "" ? (accessor + "." + key) : key));
                                b->appendBlock(b2);
                            }
                        }
                        else
                        {
                            b->addLine(accessor + ".initialize();");
                        }
                        return b;
                    }

                    CppBlockPtr AronObjectTypeCppSerializer::getWriteInitialTypeBlock(const std::string& accessor) const
                    {
                        CppBlockPtr b = CppBlockPtr(new CppBlock());

                        if (accessor == "")
                        {
                            if (typenavigator->getExtends() != nullptr)
                            {
                                const auto extends_s = FromAronTypeNaviagtorPtr(typenavigator->getExtends());
                                b->addLine(extends_s->getCppTypename() + "::writeInitialType(w);");
                            }

                            b->addLine("w.writeStartObjectType();");
                            b->addLine("w.writeObjectName(\"" + typenavigator->getObjectName() + "\");");
                            for (const auto& [key, child] : typenavigator->getAcceptedTypes())
                            {
                                const auto child_s = FromAronTypeNaviagtorPtr(child);
                                b->addLine("w.writeKey(\"" + key + "\");");
                                CppBlockPtr b2 = child_s->getWriteInitialTypeBlock(child_s->getCppTypename());
                                b->appendBlock(b2);
                            }
                            b->addLine("w.writeEndObjectType();");
                        }
                        else
                        {
                            b->addLine(accessor + "::writeInitialType(w);");
                        }
                        return b;
                    }

                    CppBlockPtr AronObjectTypeCppSerializer::getWriteCurrentTypeBlock(const std::string& accessor) const
                    {
                        CppBlockPtr b = CppBlockPtr(new CppBlock());

                        if (accessor == "")
                        {
                            if (typenavigator->getExtends() != nullptr)
                            {
                                const auto extends_s = FromAronTypeNaviagtorPtr(typenavigator->getExtends());
                                //b->addLine(extends_s->getCppTypename() + ".writeCurrentType(w);"); TODO
                            }

                            b->addLine("w.writeStartObjectType();");
                            b->addLine("w.writeObjectName(\"" + typenavigator->getObjectName() + "\");");
                            for (const auto& [key, child] : typenavigator->getAcceptedTypes())
                            {
                                const auto child_s = FromAronTypeNaviagtorPtr(child);
                                b->addLine("w.writeKey(\"" + key + "\");");
                                CppBlockPtr b2 = child_s->getWriteCurrentTypeBlock(child_s->getCppTypename());
                                b->appendBlock(b2);
                            }
                            b->addLine("w.writeEndObjectType();");
                        }
                        else
                        {
                            b->addLine(accessor + ".writeCurrentType(w);");
                        }
                        return b;
                    }

                    CppBlockPtr AronObjectTypeCppSerializer::getWriteBlock(const std::string& accessor) const
                    {
                        CppBlockPtr b = CppBlockPtr(new CppBlock());

                        if (accessor == "")
                        {
                            if (typenavigator->getExtends() != nullptr)
                            {
                                const auto extends_s = FromAronTypeNaviagtorPtr(typenavigator->getExtends());
                                b->addLine(extends_s->getCppTypename() + "::write(w, aron_type);");
                            }

                            b->addLine("w.writeStartDict();");
                            for (const auto& [key, child] : typenavigator->getAcceptedTypes())
                            {
                                const auto child_s = FromAronTypeNaviagtorPtr(child);
                                b->addLine("w.writeKey(\"" + key + "\");");
                                CppBlockPtr b2 = child_s->getWriteBlock((accessor != "" ? (accessor + "." + key) : key));
                                b->appendBlock(b2);
                            }
                            b->addLine("w.writeEndDict();");
                        }
                        else
                        {
                            b->addLine(accessor + ".write(w);");
                        }
                        return b;
                    }

                    CppBlockPtr AronObjectTypeCppSerializer::getReadBlock(const std::string& accessor) const
                    {
                        CppBlockPtr b = CppBlockPtr(new CppBlock());

                        if (accessor == "")
                        {
                            if (typenavigator->getExtends() != nullptr)
                            {
                                const auto extends_s = FromAronTypeNaviagtorPtr(typenavigator->getExtends());
                                b->addLine(extends_s->getCppTypename() + "::read(r);");
                            }

                            b->addLine("r.readStartDict();");
                            for (const auto& [key, child] : typenavigator->getAcceptedTypes())
                            {
                                const auto child_s = FromAronTypeNaviagtorPtr(child);
                                b->addLine("r.readKey(\"" + key + "\");");
                                CppBlockPtr b2 = child_s->getReadBlock((accessor != "" ? (accessor + "." + key) : key));
                                b->appendBlock(b2);
                            }
                            b->addLine("r.readEndDict();");
                        }
                        else
                        {
                            b->addLine(accessor + ".read(r);");
                        }
                        return b;
                    }

                    CppBlockPtr AronObjectTypeCppSerializer::getEqualsBlock(const std::string& accessor, const std::string& otherInstanceAccessor) const
                    {
                        CppBlockPtr b = CppBlockPtr(new CppBlock());
                        if (accessor == "") // object definition, so we are no member of some other class
                        {
                            if (typenavigator->getExtends() != nullptr)
                            {
                                const auto extends_s = FromAronTypeNaviagtorPtr(typenavigator->getExtends());
                                b->addLine("if (not (" + extends_s->getCppTypename() + "::operator== (" + otherInstanceAccessor + ")))");
                                b->addLine("\t return false;");
                            }
                            for (const auto& [key, child] : typenavigator->getAcceptedTypes())
                            {
                                auto child_s = FromAronTypeNaviagtorPtr(child);
                                CppBlockPtr b2 = child_s->getEqualsBlock(key, otherInstanceAccessor + "." + key);
                                b->appendBlock(b2);
                            }
                        }
                        else // object usage with existing operator==
                        {
                            b->addLine("if (not (" + accessor + " == " + otherInstanceAccessor + "))");
                            b->addLine("\t return false;");
                        }
                        return b;
                    }
                }
            }
        }
    }
}

