/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>
#include <stack>

// Base Class
#include <RobotAPI/libraries/aron/aroncore/io/AronDataIO/AronDataReader.h>



namespace armarx
{
    namespace aron
    {
        namespace io
        {
            template <typename InputTypename, typename TokenPtrTypename>
            class AronDataClassReader;

            template <typename InputTypename, typename TokenPtrTypename>
            using AronDataClassReaderPtr = std::shared_ptr<AronDataClassReader<InputTypename, TokenPtrTypename>>;

            template <typename InputTypename, typename TokenPtrTypename>
            class AronDataClassReader :
                virtual public AronDataReader
            {
            public:
                AronDataClassReader() = delete;
                AronDataClassReader(const InputTypename& input) :
                    data(input)
                {
                }

                // ReadKey
                std::string readKey() override
                {
                    TokenPtrTypename token = stack.top();
                    std::string k = token->resolveCurrentKey();
                    return k;
                }

                bool readKey(const std::string& k) override
                {
                    TokenPtrTypename token = stack.top();
                    token->setCurrentKey(k);
                    return true;
                }

                AronDataDescriptor getTypeOfNextAron(const AronTypeDescriptor hint) const override
                {
                    if (stack.empty())
                    {
                        // completely unknown. We assume a dict on the top level
                        return eAronDict;
                    }

                    TokenPtrTypename lastToken = stack.top();
                    return lastToken->getTypeOfNextElement(hint);
                }


            protected:
                InputTypename data;
                bool readInitialStart = false;
                std::stack<TokenPtrTypename> stack;
            };
        }
    }
}
