/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// Base Class
#include <RobotAPI/libraries/aron/aroncore/io/AronDataIO/classReaders/AronDataClassReaderToken.h>

// ArmarX
#include <RobotAPI/libraries/aron/aroncore/AronConfig.h>
#include <RobotAPI/libraries/aron/aroncore/navigators/datanavigator/AronAllDataNavigators.h>

namespace armarx
{
    namespace aron
    {
        namespace io
        {
            // BaseClass
            class AronDataNavigatorReaderToken;
            typedef std::shared_ptr<AronDataNavigatorReaderToken> AronDataNavigatorReaderTokenPtr;

            class AronDataNavigatorReaderToken :
                virtual public AronDataClassReaderToken<datanavigator::AronDataNavigatorPtr, datanavigator::AronContainerDataNavigatorPtr>
            {
            public:
                using PointerType = AronDataNavigatorReaderTokenPtr;

            public:
                // constructors
                AronDataNavigatorReaderToken() = delete;
                AronDataNavigatorReaderToken(const AronDataDescriptor desc, const datanavigator::AronContainerDataNavigatorPtr& data) :
                    AronReaderToken<AronDataDescriptor, datanavigator::AronDataNavigatorPtr, datanavigator::AronContainerDataNavigatorPtr>(desc, data),
                    AronDataClassReaderToken<datanavigator::AronDataNavigatorPtr, datanavigator::AronContainerDataNavigatorPtr>(desc, data)
                {
                    switch (descriptor)
                    {
                        case eAronDict:
                        {
                            allKeys = datanavigator::AronDictDataNavigator::DynamicCastAndCheck(data)->getAllKeys();
                            maxIndex = allKeys.size();
                            break;
                        }
                        case eAronList:
                        {
                            maxIndex = datanavigator::AronListDataNavigator::DynamicCastAndCheck(data)->childrenSize();
                            break;
                        }
                        default:
                            throw exception::AronDataDescriptorNotValidException("AronDataNavigatorReaderToken", "AronDataNavigatorReaderToken", "Received an invalid instance for a reader token.", desc);
                    }
                }

                AronDataDescriptor getTypeOfNextElement(const AronTypeDescriptor) const override
                {
                    const datanavigator::AronDataNavigatorPtr next = getCurrentElement();
                    return next->getDescriptor();
                }

                datanavigator::AronDataNavigatorPtr getCurrentElement() const override
                {
                    switch (descriptor)
                    {
                        case eAronDict:
                        {
                            datanavigator::AronDictDataNavigatorPtr casted = datanavigator::AronDictDataNavigator::DynamicCast(element);
                            datanavigator::AronDataNavigatorPtr ret = casted->getElement(resolveCurrentKey());
                            return ret;
                        }
                        case eAronList:
                        {
                            datanavigator::AronListDataNavigatorPtr casted = datanavigator::AronListDataNavigator::DynamicCast(element);
                            datanavigator::AronDataNavigatorPtr ret = casted->getElement(currentIndex);
                            return ret;
                        }
                        default:
                            throw exception::AronDataDescriptorNotValidException("AronDataNavigatorReaderToken", "getNextElement", "Could not resove a type of a navigator. Allowed are only container types due to performance", descriptor);
                    }
                }

            private:
            };
        }
    }
}
