/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <stack>

// BaseClass
#include <RobotAPI/libraries/aron/aroncore/io/AronDataIO/classReaders/AronDataClassReader.h>

// ArmarX
#include <RobotAPI/libraries/aron/aroncore/io/AronDataIO/classReaders/NlohmannJSONReader/AronDataNlohmannJSONReaderToken.h>

namespace armarx
{
    namespace aron
    {
        namespace io
        {
            class AronDataNlohmannJSONReader;
            typedef std::shared_ptr<AronDataNlohmannJSONReader> AronDataNlohmannJSONReaderPtr;

            class AronDataNlohmannJSONReader :
                virtual public AronDataClassReader<nlohmann::json, AronDataNlohmannJSONReaderTokenPtr>
            {
            public:
                using PointerType = AronDataNlohmannJSONReaderPtr;

            public:
                // constructors
                AronDataNlohmannJSONReader() = delete;
                AronDataNlohmannJSONReader(const nlohmann::json& n);
                AronDataNlohmannJSONReader(const std::string& n);

#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    virtual int  readStart##upperType() override; \
    virtual bool readEnd##upperType() override; \

                HANDLE_CONTAINER_DATA
#undef RUN_ARON_MACRO

#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    virtual std::pair<std::vector<int>, std::string> readStart##upperType() override; \
    virtual void readEnd##upperType(unsigned char*) override; \

                HANDLE_COMPLEX_DATA
#undef RUN_ARON_MACRO

#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    virtual void readPrimitive(lowerType&) override; \

                HANDLE_PRIMITIVE_TYPES
#undef RUN_ARON_MACRO

            private:
                nlohmann::json getCurrent();
                nlohmann::json getCurrentAndGoToNext();

            private:
            };
        }
    }
}
