/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// Base Class
#include <RobotAPI/libraries/aron/aroncore/io/AronWriterToken.h>

// ArmarX
#include <RobotAPI/libraries/aron/aroncore/AronConfig.h>
#include <RobotAPI/libraries/aron/aroncore/AronDescriptor.h>

namespace armarx
{
    namespace aron
    {
        namespace io
        {
            template<typename ElementTypename, typename ElementAddTypename>
            class AronDataClassWriterToken;

            template<typename ElementTypename, typename ElementAddTypename>
            using AronDataClassWriterTokenPtr = std::shared_ptr<AronDataClassWriterToken<ElementTypename, ElementAddTypename>>;

            template<typename ElementTypename, typename ElementAddTypename>
            class AronDataClassWriterToken :
                virtual public AronWriterToken<AronDataDescriptor, ElementTypename, ElementAddTypename>
            {
            public:
                using PointerType = AronDataClassWriterToken<ElementTypename, ElementAddTypename>;

            public:
                // constructor
                AronDataClassWriterToken() = delete;
                AronDataClassWriterToken(const AronDataDescriptor desc, const ElementTypename& n) :
                    AronWriterToken<AronDataDescriptor, ElementTypename, ElementAddTypename>(desc, n)
                {}

                // public member functions
                virtual void addElement(const ElementAddTypename&) override = 0;

                virtual std::string toElementAccessor() const override
                {
                    switch (this->descriptor)
                    {
                        case eAronDict:
                            return this->currentKey;
                        case eAronList:
                            return std::to_string(this->currentIndex);
                        default:
                            throw exception::AronDataDescriptorNotValidException("AronDataClassWriterToken", "toElementAccessor", "Could not resove a type of a navigator. Allowed are only container and complex types due to performance.", this->descriptor);
                    }
                }

            private:
            };
        }
    }
}
