/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller (fabian dot peller at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include <numeric>

#include "AronDataNlohmannJSONWriter.h"

namespace armarx
{
    namespace aron
    {
        namespace io
        {

            // Containers
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    bool AronDataNlohmannJSONWriter::writeStart##upperType() \
    { \
        nlohmann::json data; \
        AronDataNlohmannJSONWriterTokenPtr new_token = AronDataNlohmannJSONWriterTokenPtr(new AronDataNlohmannJSONWriterToken(AronDataDescriptor::eAron##upperType, data)); \
        stack.push(new_token); \
        return true; \
    } \
    \
    bool AronDataNlohmannJSONWriter::writeEnd##upperType() \
    { \
        lastRemovedToken = stack.top(); \
        stack.pop(); \
        \
        if (stack.size() > 0) \
        { \
            AronDataNlohmannJSONWriterTokenPtr prevToken = stack.top(); \
            prevToken->addElement(lastRemovedToken->getElement()); \
        } \
        return true; \
    }

            HANDLE_CONTAINER_DATA
#undef RUN_ARON_MACRO

            // Complex Types
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    bool AronDataNlohmannJSONWriter::write##upperType(const std::vector<int>& dims, const std::string& t, const unsigned char* data) \
    { \
        AronDataNlohmannJSONWriterTokenPtr token = stack.top(); \
        nlohmann::json j; \
        j[ARON_DATA_READER_WRITER_NDARRAY_DIMENSIONS_SLUG] = dims; \
        j[ARON_DATA_READER_WRITER_NDARRAY_TYPE_SLUG] = t; \
        int elements = std::accumulate(std::begin(dims), std::end(dims), 1, std::multiplies<int>()); \
        std::vector<unsigned char> d = std::vector<unsigned char>(elements); \
        memcpy(d.data(), data, elements); \
        j[ARON_DATA_READER_WRITER_NDARRAY_DATA_SLUG] = d; \
        token->addElement(j); \
        return true; \
    }

            HANDLE_COMPLEX_DATA
#undef RUN_ARON_MACRO

#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    bool AronDataNlohmannJSONWriter::writePrimitive(const lowerType& x) \
    { \
        AronDataNlohmannJSONWriterTokenPtr token = stack.top(); \
        nlohmann::json j(x); \
        token->addElement(j); \
        return true; \
    }

            HANDLE_PRIMITIVE_TYPES
#undef RUN_ARON_MACRO

        }
    }
}
