/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// Simox
#include <SimoxUtility/json.h>

// Base Class
#include <RobotAPI/libraries/aron/aroncore/io/AronDataIO/classWriters/AronDataClassWriterToken.h>

// ArmarX
#include <RobotAPI/libraries/aron/aroncore/AronConfig.h>

namespace armarx
{
    namespace aron
    {
        namespace io
        {
            class AronDataNlohmannJSONWriterToken;
            typedef std::shared_ptr<AronDataNlohmannJSONWriterToken> AronDataNlohmannJSONWriterTokenPtr;

            class AronDataNlohmannJSONWriterToken :
                virtual public AronDataClassWriterToken<nlohmann::json, nlohmann::json>
            {
            public:
                using PointerType = AronDataNlohmannJSONWriterTokenPtr;

            public:
                // constructor
                AronDataNlohmannJSONWriterToken() = delete;
                AronDataNlohmannJSONWriterToken(const AronDataDescriptor desc, const nlohmann::json& d) :
                    AronWriterToken<AronDataDescriptor, nlohmann::json, nlohmann::json>(desc, d),
                    AronDataClassWriterToken<nlohmann::json, nlohmann::json>(desc, d)
                {

                }

                // public member functions
                virtual void addElement(const nlohmann::json& n) override
                {
                    auto desc = this->getDescriptor();
                    switch (desc)
                    {
                        case eAronDict:
                        {
                            element[currentKey] = n;
                            break;
                        }
                        case eAronList:
                        {
                            element.push_back(n);
                            currentIndex++;
                            break;
                        }
                        default:
                            throw exception::AronDataDescriptorNotValidException("AronDataNlohmannJSONWriterToken", "addElement", "Could not resove a type of a token. Allowed are only containers.", desc);
                    }
                }

            private:
            };
        }
    }
}
