/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// Simox
#include <SimoxUtility/xml.h>

// Base Class
#include <RobotAPI/libraries/aron/aroncore/io/AronDataIO/classWriters/AronDataClassWriterToken.h>

// ArmarX
#include <ArmarXCore/core/rapidxml/wrapper/RapidXmlReader.h>
#include <RobotAPI/libraries/aron/aroncore/AronConfig.h>

namespace armarx
{
    namespace aron
    {
        namespace io
        {
            class AronDataRapidXMLWriterToken;
            typedef std::shared_ptr<AronDataRapidXMLWriterToken> AronDataRapidXMLWriterTokenPtr;

            class AronDataRapidXMLWriterToken :
                virtual public AronDataClassWriterToken<RapidXmlReaderNode, RapidXmlReaderNode>
            {
            public:
                using PointerType = AronDataRapidXMLWriterTokenPtr;

            public:
                // constructor
                AronDataRapidXMLWriterToken() = delete;
                AronDataRapidXMLWriterToken(const AronDataDescriptor desc, const RapidXmlReaderNode& d) :
                    AronWriterToken<AronDataDescriptor, RapidXmlReaderNode, RapidXmlReaderNode>(desc, d),
                    AronDataClassWriterToken<RapidXmlReaderNode, RapidXmlReaderNode>(desc, d)
                {

                }

                // public member functions
                virtual void addElement(const RapidXmlReaderNode& n) override
                {
                    auto desc = this->getDescriptor();
                    switch (desc)
                    {
                        case eAronDict:
                        {
                            break;
                        }
                        case eAronList:
                        {
                            break;
                        }
                        default:
                            throw exception::AronDataDescriptorNotValidException("AronDataRapidXMLWriterToken", "addElement", "Could not resove a type of a token. Allowed are only containers.", desc);
                    }
                }

            private:
            };
        }
    }
}
