/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

// STD/STL
#include <memory>
#include <numeric>

// Header
#include "AronTypeConverter.h"




namespace armarx
{
    namespace aron
    {
        namespace io
        {
            void AronTypeConverter::ConvertFromReader(AronTypeReader& reader, AronTypeWriter& writer)
            {
                AronTypeDescriptor desc = reader.getTypeOfNextAron();
                switch (desc)
                {
                    case eAronObjectType:
                    {
                        const int elements = reader.readStartObjectType();
                        const std::string name = reader.readObjectName();

                        writer.writeStartObjectType();
                        writer.writeObjectName(name);

                        for (int i = 0; i < elements; ++i)
                        {
                            std::string key = reader.readKey();
                            writer.writeKey(key);
                            AronTypeConverter::ConvertFromReader(reader, writer);
                        }

                        writer.writeEndObjectType();
                        reader.readEndObjectType();
                        break;
                    }
                    case eAronDictType:
                    {
                        reader.readStartDictType();
                        writer.writeStartDictType();

                        AronTypeConverter::ConvertFromReader(reader, writer);

                        writer.writeEndDictType();
                        reader.readEndDictType();
                        break;
                    }
                    case eAronTupleType:
                    {
                        int elements = reader.readStartTupleType();
                        writer.writeStartTupleType();

                        for (int i = 0; i < elements; ++i)
                        {
                            AronTypeConverter::ConvertFromReader(reader, writer);
                        }

                        writer.writeEndTupleType();
                        reader.readEndTupleType();
                        break;
                    }
                    case eAronListType:
                    {
                        reader.readStartListType();
                        writer.writeStartListType();

                        AronTypeConverter::ConvertFromReader(reader, writer);

                        writer.writeEndListType();
                        reader.readEndListType();
                        break;
                    }
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
case eAron##upperType##Type: \
{ \
    auto [dims, type] = reader.read##upperType##Type(); \
    writer.write##upperType##Type(dims, type); \
    break; \
}

                    HANDLE_COMPLEX_TYPES
#undef RUN_ARON_MACRO

#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
case eAron##upperType##Type: \
{ \
    reader.read##upperType##Type(); \
    writer.write##upperType##Type(); \
    break; \
}

                    HANDLE_PRIMITIVE_TYPES
#undef RUN_ARON_MACRO

                    default:
                    {
                        throw exception::AronTypeDescriptorNotValidException("LegacyAronTypeReader", "SetupReaderAndGetResult", "Type-Type could not be resolved", desc);
                    }
                }
            }
        }
    }
}
