/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// BaseClass
#include <RobotAPI/libraries/aron/aroncore/io/AronTypeIO/AronTypeReaderWriter.h>

// ArmarX
#include <ArmarXCore/core/exceptions/Exception.h>
#include <RobotAPI/libraries/aron/aroncore/AronConfig.h>
#include <RobotAPI/libraries/aron/aroncore/io/AronTypeIO/classWriters/AronTypeClassWriter.h>


namespace armarx
{
    namespace aron
    {
        namespace io
        {
            class AronTypeReader;
            typedef std::shared_ptr<AronTypeReader> AronTypeReaderPtr;

            class AronTypeReader :
                public AronTypeReaderWriter
            {
            public:
                AronTypeReader() = default;

#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    virtual int readStart##upperType##Type() = 0; \
    virtual bool readEnd##upperType##Type() = 0; \

                HANDLE_CONTAINER_TYPES
#undef RUN_ARON_MACRO

#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    virtual std::pair<std::vector<int>, std::string> read##upperType##Type() = 0; \

                HANDLE_COMPLEX_TYPES
#undef RUN_ARON_MACRO

#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    virtual void read##upperType##Type() = 0; \

                HANDLE_PRIMITIVE_TYPES
#undef RUN_ARON_MACRO

                virtual std::string readKey() = 0;
                virtual bool readKey(const std::string&) = 0;
                virtual std::string readObjectName() = 0;

                // Helper functions
                virtual AronTypeDescriptor getTypeOfNextAron() const = 0;
            };
        }
    }
}
