/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// ArmarX
#include <RobotAPI/libraries/aron/aroncore/AronException.h>

#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/aroncore/io/AronReaderWriter.h>


namespace armarx
{
    namespace aron
    {
        namespace io
        {
            class AronTypeReaderWriter;
            typedef std::shared_ptr<AronTypeReaderWriter> AronTypeReaderWriterPtr;

            class AronTypeReaderWriter :
                public AronReaderWriter
            {
            public:
                AronTypeReaderWriter() = default;

            protected:
                const std::string ARON_TYPE_READER_WRITER_NDARRAY_NAME_SLUG = "ARON_NDARRAY_NAME";
                const std::string ARON_TYPE_READER_WRITER_NDARRAY_DIMENSIONS_SLUG = "ARON_NDARRAY_DIMESIONS";
                const std::string ARON_TYPE_READER_WRITER_NDARRAY_TYPE_SLUG = "ARON_NDARRAY_TYPE";
                const std::string ARON_TYPE_READER_WRITER_NDARRAY_DATA_SLUG = "ARON_NDARRAY_DATA";

#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    const std::string ARON_TYPE_READER_WRITER_##capsType##_TYPENAME = "ARON_" + std::string(#capsType) + "_TYPE";

                HANDLE_PRIMITIVE_TYPES
#undef RUN_ARON_MACRO
            };
        }
    }
}
