/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

// STD/STL
#include <memory>
#include <numeric>

// Header
#include "AronTypeWriterVisitor.h"

// ArmarX
#include <RobotAPI/libraries/aron/aroncore/AronResolver.h>



namespace armarx
{
    namespace aron
    {
        namespace io
        {

            void AronTypeWriterVisitor::SetupWriterFromAronTypePtr(AronTypeWriter& writer, const typenavigator::AronTypeNavigatorPtr& aron)
            {
                SetupWriterFromAronTypePtr(writer, aron->getResult());
            }

            void AronTypeWriterVisitor::SetupWriterFromAronTypePtr(AronTypeWriter& writer, const type::AronTypePtr& aron)
            {
                AronTypeDescriptor desc = AronResolver::GetDescriptorForAronType(aron);
                switch (desc)
                {
                    case eAronObjectType:
                    {
                        type::AronObjectTypePtr casted = type::AronObjectTypePtr::dynamicCast(aron);
                        writer.writeStartObjectType();
                        writer.writeObjectName(casted->objectName);
                        for (const auto& [key, value] : casted->elementTypes)
                        {
                            writer.writeKey(key);
                            AronTypeWriterVisitor::SetupWriterFromAronTypePtr(writer, value);
                        }
                        writer.writeEndObjectType();
                        break;
                    }
                    case eAronDictType:
                    {
                        type::AronDictTypePtr casted = type::AronDictTypePtr::dynamicCast(aron);
                        writer.writeStartDictType();
                        AronTypeWriterVisitor::SetupWriterFromAronTypePtr(writer, casted->acceptedType);
                        writer.writeEndDictType();
                        break;
                    }
                    case eAronTupleType:
                    {
                        type::AronTupleTypePtr casted = type::AronTupleTypePtr::dynamicCast(aron);
                        writer.writeStartTupleType();
                        for (const auto& value : casted->elementTypes)
                        {
                            AronTypeWriterVisitor::SetupWriterFromAronTypePtr(writer, value);
                        }
                        writer.writeEndTupleType();
                        break;
                    }
                    case eAronListType:
                    {
                        type::AronListTypePtr casted = type::AronListTypePtr::dynamicCast(aron);
                        writer.writeStartListType();
                        AronTypeWriterVisitor::SetupWriterFromAronTypePtr(writer, casted->acceptedType);
                        writer.writeEndListType();
                        break;
                    }
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
case eAron##upperType##Type: \
{ \
    type::Aron##upperType##TypePtr casted = type::Aron##upperType##TypePtr::dynamicCast(aron); \
    writer.write##upperType##Type(casted->dimensions, casted->typeName); \
    break; \
}

                    HANDLE_COMPLEX_TYPES
#undef RUN_ARON_MACRO

#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
case eAron##upperType##Type: \
{ \
    type::Aron##upperType##TypePtr casted = type::Aron##upperType##TypePtr::dynamicCast(aron); \
    writer.write##upperType##Type(); \
    break; \
}

                    HANDLE_PRIMITIVE_TYPES
#undef RUN_ARON_MACRO

                    default:
                    {
                        throw exception::AronTypeDescriptorNotValidException("LegacyAronTypeWriter", "SetupWriterFromAronTypePtr", "Type-Type could not be resolved. The ice_id of the input was: " + aron->ice_id(), desc);
                    }
                }
            }

        }
    }
}
