/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// Base Class
#include <RobotAPI/libraries/aron/aroncore/io/AronReaderToken.h>

// ArmarX
#include <RobotAPI/libraries/aron/aroncore/AronConfig.h>
#include <RobotAPI/libraries/aron/aroncore/AronDescriptor.h>



namespace armarx
{
    namespace aron
    {
        namespace io
        {
            template<typename ElementReturnPtrTypename, typename ElementAddTypename>
            class AronTypeClassReaderToken;

            template<typename ElementReturnPtrTypename, typename ElementAddTypename>
            using AronTypeClassReaderTokenPtr = std::shared_ptr<AronTypeClassReaderToken<ElementReturnPtrTypename, ElementAddTypename>>;

            template<typename ElementReturnPtrTypename, typename ElementAddTypename>
            class AronTypeClassReaderToken :
                virtual public AronReaderToken<AronTypeDescriptor, ElementReturnPtrTypename, ElementAddTypename>
            {
            public:
                using PointerType = AronTypeClassReaderTokenPtr<ElementReturnPtrTypename, ElementAddTypename>;

            public:
                // constructors
                AronTypeClassReaderToken() = delete;
                AronTypeClassReaderToken(const AronTypeDescriptor desc, const ElementAddTypename& type) :
                    AronReaderToken<AronTypeDescriptor, ElementReturnPtrTypename, ElementAddTypename>(desc, type)
                { }

                bool currentIsEligableForResolvingKey() const override
                {
                    if (this->descriptor == AronTypeDescriptor::eAronObjectType)
                    {
                        return true;
                    }
                    return false;
                }

                virtual std::string getObjectName() const = 0;
                virtual AronTypeDescriptor getTypeOfNextElement() const = 0;

            private:
            };
        }
    }
}
