/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <stack>

// BaseClass
#include <RobotAPI/libraries/aron/aroncore/io/AronTypeIO/classReaders/AronTypeClassReader.h>

// ArmarX
#include <RobotAPI/libraries/aron/aroncore/navigators/typenavigator/AronTypeNavigator.h>
#include <RobotAPI/libraries/aron/aroncore/io/AronTypeIO/classReaders/NavigatorReader/AronTypeNavigatorReaderToken.h>


namespace armarx
{
    namespace aron
    {
        namespace io
        {
            class AronTypeNavigatorReader;
            typedef std::shared_ptr<AronTypeNavigatorReader> AronTypeNavigatorReaderPtr;

            class AronTypeNavigatorReader :
                virtual public AronTypeClassReader<typenavigator::AronContainerTypeNavigatorPtr, AronTypeNavigatorReaderTokenPtr>
            {
            public:
                using PointerType = AronTypeNavigatorReaderPtr;

            public:
                // constructors
                AronTypeNavigatorReader() = delete;
                AronTypeNavigatorReader(const typenavigator::AronContainerTypeNavigatorPtr& n);
                AronTypeNavigatorReader(const type::AronContainerTypePtr& n);

#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    virtual int readStart##upperType##Type() override; \
    virtual bool readEnd##upperType##Type() override; \

                HANDLE_CONTAINER_TYPES
#undef RUN_ARON_MACRO

#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    virtual std::pair<std::vector<int>, std::string> read##upperType##Type() override; \

                HANDLE_COMPLEX_TYPES
#undef RUN_ARON_MACRO

#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    virtual void read##upperType##Type() override; \

                HANDLE_PRIMITIVE_TYPES
#undef RUN_ARON_MACRO

            private:
                typenavigator::AronTypeNavigatorPtr getCurrentAndGoToNext();
                typenavigator::AronTypeNavigatorPtr getCurrent();

            private:
            };
        }
    }
}
