/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>
#include <stack>

// ArmarX
#include <RobotAPI/libraries/aron/aroncore/io/AronTypeIO/AronTypeWriter.h>



namespace armarx
{
    namespace aron
    {
        namespace io
        {
            template <typename GeneratePtr, typename TokenPtr>
            class AronTypeClassWriter;

            template <typename GeneratePtr, typename TokenPtr>
            using AronTypeClassWriterPtr = std::shared_ptr<AronTypeClassWriter<GeneratePtr, TokenPtr>>;

            template <typename GeneratePtr, typename TokenPtr>
            class AronTypeClassWriter :
                virtual public AronTypeWriter
            {
            public:
                AronTypeClassWriter() = default;

                virtual bool writeKey(const std::string& k) override
                {
                    TokenPtr token = stack.top();
                    token->setCurrentKey(k);
                    return true;
                }

                bool writeObjectName(const std::string& n)
                {
                    TokenPtr token = stack.top();
                    token->setName(n);
                    return true;
                }

                GeneratePtr getResult() const
                {
                    assertLastRemovedTokenIsSet();
                    assertStackIsEmpty();
                    return lastRemovedToken->getElement();
                }

            protected:
                void assertLastRemovedTokenIsSet() const
                {
                    if (lastRemovedToken == nullptr)
                    {
                        throw exception::AronException("AronTypeClassWriter", "assertLastRemovedTokenIsSet", "lastRemovedToken is NULL. Perhaps you never called a writeEnd* method?");
                    }
                    if (lastRemovedToken->getElement() == nullptr)
                    {
                        throw exception::AronException("AronTypeClassWriter", "assertLastRemovedTokenIsSet", "lastRemovedToken->getElement() is NULL.");
                    }
                }
                void assertStackIsEmpty() const
                {
                    if (stack.size() != 0)
                    {
                        throw exception::AronException("AronTypeClassWriter", "assertStackIsEmpty", "The Size of the stack is not 0. Perhaps you never called a writeEnd* method?");
                    }
                }

            protected:
                bool wroteInitialStartContainer = false;
                TokenPtr lastRemovedToken;
                std::stack<TokenPtr> stack;
            };
        }
    }
}
