/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// Base Class
#include <RobotAPI/libraries/aron/aroncore/io/AronWriterToken.h>

// ArmarX
#include <RobotAPI/libraries/aron/aroncore/AronConfig.h>

namespace armarx
{
    namespace aron
    {
        namespace io
        {
            template<typename ElementPtrTypename, typename ElementAddTypename>
            class AronTypeClassWriterToken;

            template<typename ElementPtrTypename, typename ElementAddTypename>
            using AronTypeClassWriterTokenPtr = std::shared_ptr<AronTypeClassWriterToken<ElementPtrTypename, ElementAddTypename>>;

            template<typename ElementPtrTypename, typename ElementAddTypename>
            class AronTypeClassWriterToken :
                virtual public AronWriterToken<AronTypeDescriptor, ElementPtrTypename, ElementAddTypename>
            {
            public:
                using PointerType = AronTypeClassWriterToken<ElementPtrTypename, ElementAddTypename>;

            public:
                // constructor
                AronTypeClassWriterToken() = delete;
                AronTypeClassWriterToken(const AronTypeDescriptor desc, const ElementPtrTypename& n) :
                    AronWriterToken<AronTypeDescriptor, ElementPtrTypename, ElementAddTypename>(desc, n)
                {}

                // public member functions
                virtual void addElement(const ElementAddTypename&) = 0;

                virtual std::string toElementAccessor() const override
                {
                    AronTypeDescriptor desc = this->getDescriptor();
                    switch (desc)
                    {
                        case eAronDictType:
                        case eAronListType:
                            return "[ACCEPTED_TYPE]";
                        case eAronObjectType:
                            return "\"" + this->currentKey + "\"";
                        case eAronTupleType:
                            return "<" + std::to_string(this->currentIndex) + ">";
                        default:
                            throw exception::AronTypeDescriptorNotValidException("AronTypeNavigatorWriterToken", "toElementAccessor", "Could not resove a type of a navigator. Allowed are only containers.", desc);
                    }
                }

                virtual void setName(const std::string& n) = 0;

            private:
            };
        }
    }
}
