/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// Base Class
#include <RobotAPI/libraries/aron/aroncore/io/AronTypeIO/classWriters/AronTypeClassWriterToken.h>

// ArmarX
#include <RobotAPI/libraries/aron/aroncore/AronConfig.h>
#include <RobotAPI/libraries/aron/aroncore/navigators/typenavigator/AronAllTypeNavigators.h>

namespace armarx
{
    namespace aron
    {
        namespace io
        {
            class AronTypeNavigatorWriterToken;
            typedef std::shared_ptr<AronTypeNavigatorWriterToken> AronTypeNavigatorWriterTokenPtr;

            class AronTypeNavigatorWriterToken :
                virtual public AronTypeClassWriterToken<typenavigator::AronContainerTypeNavigatorPtr, typenavigator::AronTypeNavigatorPtr>
            {
            public:
                using PointerType = AronTypeNavigatorWriterTokenPtr;

            public:
                // constructor
                AronTypeNavigatorWriterToken() = delete;
                AronTypeNavigatorWriterToken(const AronTypeDescriptor desc, const typenavigator::AronContainerTypeNavigatorPtr& t) :
                    AronWriterToken<AronTypeDescriptor, typenavigator::AronContainerTypeNavigatorPtr, typenavigator::AronTypeNavigatorPtr>(desc, t),
                    AronTypeClassWriterToken<typenavigator::AronContainerTypeNavigatorPtr, typenavigator::AronTypeNavigatorPtr>(desc, t)
                {
                }

                // virtual member functions
                virtual void addElement(const typenavigator::AronTypeNavigatorPtr& n) override
                {
                    switch (descriptor)
                    {
                        case eAronDictType:
                        {
                            typenavigator::AronDictTypeNavigatorPtr casted = typenavigator::AronDictTypeNavigator::DynamicCastAndCheck(element);
                            casted->setAcceptedType(n);
                            break;
                        }
                        case eAronListType:
                        {
                            typenavigator::AronListTypeNavigatorPtr casted = typenavigator::AronListTypeNavigator::DynamicCastAndCheck(element);
                            casted->setAcceptedType(n);
                            break;
                        }
                        case eAronObjectType:
                        {
                            typenavigator::AronObjectTypeNavigatorPtr casted = typenavigator::AronObjectTypeNavigator::DynamicCastAndCheck(element);
                            casted->addAcceptedType(currentKey, n);
                            break;
                        }
                        case eAronTupleType:
                        {
                            typenavigator::AronTupleTypeNavigatorPtr casted = typenavigator::AronTupleTypeNavigator::DynamicCastAndCheck(element);
                            casted->addAcceptedType(n);
                            currentIndex++;
                            break;
                        }
                        default:
                            throw exception::AronTypeDescriptorNotValidException("AronTypeNavigatorWriterToken", "addElement", "Could not resove a type of a navigator. Allowed are only containers. The path was: " + getElement()->pathToString(), descriptor);
                    }
                }

                virtual void setName(const std::string& n) override
                {
                    if (descriptor != AronTypeDescriptor::eAronObjectType)
                    {
                        throw exception::AronTypeDescriptorNotValidException("AronTypeNavigatorWriterToken", "setName", "Cant set the name of a non-object token. The path was: " + getElement()->pathToString(), descriptor);
                    }
                    typenavigator::AronObjectTypeNavigatorPtr casted = typenavigator::AronObjectTypeNavigator::DynamicCast(element);
                    casted->setObjectName(n);
                }
            };
        }
    }
}
