/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller (fabian dot peller at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "AronTypeNlohmannJSONWriter.h"

namespace armarx
{
    namespace aron
    {
        namespace io
        {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    bool AronTypeNlohmannJSONWriter::writeStart##upperType##Type() \
    { \
        nlohmann::json data; \
        AronTypeNlohmannJSONWriterTokenPtr new_token = AronTypeNlohmannJSONWriterTokenPtr(new AronTypeNlohmannJSONWriterToken(AronTypeDescriptor::eAron##upperType##Type, data)); \
        stack.push(new_token); \
        return true; \
    } \
    bool AronTypeNlohmannJSONWriter::writeEnd##upperType##Type() \
    { \
        lastRemovedToken = stack.top(); \
        stack.pop(); \
        \
        if (stack.size() > 0) \
        { \
            AronTypeNlohmannJSONWriterTokenPtr prevToken = stack.top(); \
            prevToken->addElement(lastRemovedToken->getElement()); \
        } \
        return true; \
    }

            HANDLE_CONTAINER_TYPES
#undef RUN_ARON_MACRO

#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    bool AronTypeNlohmannJSONWriter::write##upperType##Type(const std::vector<int>& dims, const std::string& t) \
    { \
        AronTypeNlohmannJSONWriterTokenPtr token = stack.top(); \
        nlohmann::json j; \
        j[ARON_TYPE_READER_WRITER_NDARRAY_NAME_SLUG] = std::string(#upperType); \
        j[ARON_TYPE_READER_WRITER_NDARRAY_DIMENSIONS_SLUG] = dims; \
        j[ARON_TYPE_READER_WRITER_NDARRAY_TYPE_SLUG] = t; \
        token->addElement(j); \
        return true; \
    }

            HANDLE_COMPLEX_TYPES
#undef RUN_ARON_MACRO

#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    bool AronTypeNlohmannJSONWriter::write##upperType##Type() \
    { \
        AronTypeNlohmannJSONWriterTokenPtr token = stack.top(); \
        nlohmann::json j(ARON_TYPE_READER_WRITER_##capsType##_TYPENAME); \
        token->addElement(j); \
        return true; \
    }

            HANDLE_PRIMITIVE_TYPES
#undef RUN_ARON_MACRO

        }
    }
}
