/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// Simox
#include <SimoxUtility/json.h>

// Base Class
#include <RobotAPI/libraries/aron/aroncore/io/AronTypeIO/classWriters/AronTypeClassWriterToken.h>

// ArmarX
#include <RobotAPI/libraries/aron/aroncore/AronConfig.h>

namespace armarx
{
    namespace aron
    {
        namespace io
        {
            class AronTypeNlohmannJSONWriterToken;
            typedef std::shared_ptr<AronTypeNlohmannJSONWriterToken> AronTypeNlohmannJSONWriterTokenPtr;

            class AronTypeNlohmannJSONWriterToken :
                virtual public AronTypeClassWriterToken<nlohmann::json, nlohmann::json>
            {
            public:
                using PointerType = AronTypeNlohmannJSONWriterTokenPtr;

            public:
                // constructor
                AronTypeNlohmannJSONWriterToken() = delete;
                AronTypeNlohmannJSONWriterToken(const AronTypeDescriptor desc, const nlohmann::json& t) :
                    AronWriterToken<AronTypeDescriptor, nlohmann::json, nlohmann::json>(desc, t),
                    AronTypeClassWriterToken<nlohmann::json, nlohmann::json>(desc, t)
                {
                }

                // virtual member functions
                virtual void addElement(const nlohmann::json& n) override
                {
                    auto desc = getDescriptor();
                    switch (desc)
                    {
                        case eAronDictType:
                        {
                            element[ARON_READER_WRITER_TOKEN_DICT_ACCEPTED_TYPE_SLUG] = n;
                            break;
                        }
                        case eAronListType:
                        {
                            element[ARON_READER_WRITER_TOKEN_LIST_ACCEPTED_TYPE_SLUG] = n;
                            break;
                        }
                        case eAronObjectType:
                        {
                            element[currentKey] = n;
                            break;
                        }
                        case eAronTupleType:
                        {
                            element.push_back(n);
                            break;
                        }
                        default:
                            throw exception::AronTypeDescriptorNotValidException("AronTypeNlohmannJSONWriterToken", "addElement", "Could not resove a type of a navigator. Allowed are only containers.", desc);
                    }
                }


                // name functions
                void setName(const std::string& n) override
                {
                    auto desc = getDescriptor();
                    if (desc != AronTypeDescriptor::eAronObjectType)
                    {
                        throw exception::AronTypeDescriptorNotValidException("AronTypeNlohmannJSONWriterToken", "setName", "Cant set the name of a non-object token.", desc);
                    }
                    element[ARON_READER_WRITER_TOKEN_OBJECT_NAME_SLUG] = n;
                }
            };
        }
    }
}
