/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// Simox
#include <SimoxUtility/xml.h>

// Base Class
#include <RobotAPI/libraries/aron/aroncore/io/AronTypeIO/classWriters/AronTypeClassWriterToken.h>

// ArmarX
#include <ArmarXCore/core/rapidxml/wrapper/RapidXmlReader.h>
#include <RobotAPI/libraries/aron/aroncore/AronConfig.h>

namespace armarx
{
    namespace aron
    {
        namespace io
        {
            class AronTypeRapidXMLWriterToken;
            typedef std::shared_ptr<AronTypeRapidXMLWriterToken> AronTypeRapidXMLWriterTokenPtr;

            class AronTypeRapidXMLWriterToken :
                virtual public AronTypeClassWriterToken<RapidXmlReaderNode, RapidXmlReaderNode>
            {
            public:
                using PointerType = AronTypeRapidXMLWriterTokenPtr;

            public:
                // constructor
                AronTypeRapidXMLWriterToken() = delete;
                AronTypeRapidXMLWriterToken(const AronTypeDescriptor desc, const RapidXmlReaderNode& t) :
                    AronWriterToken<AronTypeDescriptor, RapidXmlReaderNode, RapidXmlReaderNode>(desc, t),
                    AronTypeClassWriterToken<RapidXmlReaderNode, RapidXmlReaderNode>(desc, t)
                {
                }

                // virtual member functions
                virtual void addElement(const RapidXmlReaderNode& n) override
                {
                    auto desc = getDescriptor();
                    switch (desc)
                    {
                        case eAronDictType:
                        {
                            break;
                        }
                        case eAronListType:
                        {
                            break;
                        }
                        case eAronObjectType:
                        {
                            break;
                        }
                        case eAronTupleType:
                        {
                            break;
                        }
                        default:
                            throw exception::AronTypeDescriptorNotValidException("AronTypeRapidXMLWriterToken", "addElement", "Could not resove a type of a navigator. Allowed are only containers.", desc);
                    }
                }

                // name functions
                void setName(const std::string& n) override
                {
                    auto desc = getDescriptor();
                    if (desc != AronTypeDescriptor::eAronObjectType)
                    {
                        throw exception::AronTypeDescriptorNotValidException("AronTypeRapidXMLWriterToken", "setName", "Cant set the name of a non-object token.", desc);
                    }
                }
            };
        }
    }
}
