/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// Base class
#include <RobotAPI/libraries/aron/aroncore/io/AronReaderWriterToken.h>

// ArmarX
#include <ArmarXCore/core/exceptions/Exception.h>

#include <RobotAPI/libraries/aron/aroncore/AronConfig.h>
#include <RobotAPI/libraries/aron/aroncore/AronException.h>



namespace armarx
{
    namespace aron
    {
        namespace io
        {
            template<typename AronDescriptorTypename, typename ElementTypename, typename ElementAddTypename>
            class AronWriterToken;

            template<typename AronDescriptorTypename, typename ElementTypename, typename ElementAddTypename>
            using AronWriterTokenPtr = std::shared_ptr<AronWriterToken<AronDescriptorTypename, ElementTypename, ElementAddTypename>> ;

            template<typename AronDescriptorTypename, typename ElementTypename, typename ElementAddTypename>
            class AronWriterToken :
                public AronReaderWriterToken
            {
            public:
                AronWriterToken() = delete;
                AronWriterToken(const AronDescriptorTypename& desc, const ElementTypename& e) :
                    descriptor(desc), element(e), currentIndex(0), currentKey("")
                {
                };

                // virtual definitions
                virtual void addElement(const ElementAddTypename&) = 0;
                virtual std::string toElementAccessor() const = 0;

                void setCurrentKey(const std::string& s)
                {
                    if (s.empty())
                    {
                        throw exception::AronException("AronWriterToken", "setCurrentKey", "The passed key is empty.");
                    }
                    currentKey = s;
                }

                virtual ElementTypename getElement() const
                {
                    return element;
                }

                virtual AronDescriptorTypename getDescriptor() const
                {
                    return descriptor;
                }

            protected:
                // members
                AronDescriptorTypename descriptor;
                ElementTypename element;

                // current index
                unsigned int currentIndex;

                // current key
                std::string currentKey;
            };
        }
    }
}
