/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "AronDataNavigator.h"

// ArmarX
#include <RobotAPI/libraries/aron/aroncore/AronException.h>
#include <RobotAPI/libraries/aron/aroncore/navigators/datanavigator/AronDataNavigatorFactory.h>

namespace armarx
{
    namespace aron
    {
        namespace datanavigator
        {
            // static data members
            const AronDataNavigatorFactoryPtr AronDataNavigator::FACTORY = AronDataNavigatorFactoryPtr(new AronDataNavigatorFactory());

            // constructors
            AronDataNavigator::AronDataNavigator(const AronDataDescriptor& descriptor, const AronPath& path) :
                navigator::AronNavigator<AronDataDescriptor, data::AronData>::AronNavigator(descriptor, path)
            {
            }

            // static methods
            AronDataNavigatorPtr AronDataNavigator::FromAronData(const data::AronDataPtr& a, const AronPath& path)
            {
                return FACTORY->create(a, path);
            }

            std::vector<AronDataNavigatorPtr> AronDataNavigator::FromAronData(const std::vector<data::AronDataPtr>& a, const AronPath& path)
            {
                std::vector<AronDataNavigatorPtr> ret;
                for (const auto& aron : a)
                {
                    ret.push_back(AronDataNavigator::FromAronData(aron, path));
                }
                return ret;
            }
            std::vector<data::AronDataPtr> AronDataNavigator::ToAronData(const std::vector<AronDataNavigatorPtr>& a)
            {
                std::vector<data::AronDataPtr> ret;
                for (const auto& aron : a)
                {
                    ret.push_back(aron->getResult());
                }
                return ret;
            }

            void AronDataNavigator::CheckDataNavigatorPtrForNull(const std::string& c, const std::string& m, const AronDataNavigatorPtr& data)
            {
                if (data.get() == nullptr)
                {
                    throw exception::AronException(c, m, "Could not make use of an AronDataNavigatorPtr. The Ptr was NULL");
                }
            }

            void AronDataNavigator::CheckDataNavigatorPtrForNull(const std::string& c, const std::string& m, const AronPath& p, const AronDataNavigatorPtr& data)
            {
                if (data.get() == nullptr)
                {
                    throw exception::AronExceptionWithPathInfo(c, m, "Could not make use of an AronDataNavigatorPtr. The Ptr was NULL", p);
                }
            }


            AronContainerDataNavigatorPtr AronContainerDataNavigator::DynamicCast(const AronDataNavigatorPtr& n)
            {
                AronContainerDataNavigatorPtr casted = std::dynamic_pointer_cast<AronContainerDataNavigator>(n);
                return casted;
            }

            AronContainerDataNavigatorPtr AronContainerDataNavigator::DynamicCastAndCheck(const AronDataNavigatorPtr& n)
            {
                CheckDataNavigatorPtrForNull("AronContainerDataNavigator", "DynamicCastAndCheck[Before]", n);
                AronContainerDataNavigatorPtr casted = AronContainerDataNavigator::DynamicCast(n);
                CheckDataNavigatorPtrForNull("AronContainerDataNavigator", "DynamicCastAndCheck[After]", n->getPath(), casted);
                return casted;
            }

            AronDataNavigatorPtr AronContainerDataNavigator::navigateRelative(const AronPath& path) const
            {
                AronPath absoluteFromHere = path.getWithoutPrefix(getPath());
                return navigateAbsolute(absoluteFromHere);
            }
        }
    }
}
