/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <map>

// ArmarX
#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/aroncore/navigators/datanavigator/AronDataNavigator.h>
#include <RobotAPI/libraries/aron/aroncore/navigators/typenavigator/AronDictTypeNavigator.h>

namespace armarx
{
    namespace aron
    {
        namespace datanavigator
        {
            class AronDictDataNavigator;
            typedef std::shared_ptr<AronDictDataNavigator> AronDictDataNavigatorPtr;

            class AronDictDataNavigator :
                virtual public AronContainerDataNavigator
            {
            public:
                using PointerType = AronDictDataNavigatorPtr;

            public:
                // constructors
                AronDictDataNavigator(const AronPath& path = AronPath());
                AronDictDataNavigator(const data::AronDictPtr&, const AronPath& path = AronPath());
                AronDictDataNavigator(const data::AronDataDict&, const AronPath& path = AronPath());
                AronDictDataNavigator(const std::map<std::string, AronDataNavigatorPtr>&, const AronPath& path = AronPath());

                // operators
                bool operator==(const AronDictDataNavigator&) const;
                virtual bool equalsDataNavigator(const AronDataNavigatorPtr&) const override;
                bool equalsDeep(const AronDictDataNavigatorPtr&) const;

                // static methods
                static AronDictDataNavigatorPtr DynamicCast(const AronDataNavigatorPtr& n);
                static AronDictDataNavigator DynamicCast(AronDataNavigator& n);
                static AronDictDataNavigatorPtr DynamicCastAndCheck(const AronDataNavigatorPtr& n);

                static AronDictDataNavigatorPtr FromAronDictPtr(const data::AronDictPtr& aron);
                static data::AronDictPtr ToAronDictPtr(const AronDictDataNavigatorPtr& navigator);

                // public member functions
                data::AronDictPtr toAronDictPtr() const;
                std::vector<std::string> getAllKeys() const;

                void addElement(const std::string& key, const AronDataNavigatorPtr&);
                bool hasElement(const std::string&) const;
                AronDataNavigatorPtr getElement(const std::string&) const;
                std::map<std::string, AronDataNavigatorPtr> getElements() const;

                void clear();

                // virtual implementations
                virtual data::AronDataPtr getResult() const override;
                virtual std::string getName() const override;
                virtual std::vector<AronDataNavigatorPtr> getChildren() const override;
                virtual size_t childrenSize() const override;

                virtual typenavigator::AronTypeNavigatorPtr recalculateType() const override;
                virtual bool fullfillsType(const typenavigator::AronTypeNavigatorPtr&) const override;

                virtual AronDataNavigatorPtr navigateAbsolute(const AronPath& path) const override;

            private:
                // members
                std::map<std::string, AronDataNavigatorPtr> childrenNavigators;
                data::AronDictPtr aron;
            };
        }
    }
}
