/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>

// ArmarX
#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/aroncore/navigators/datanavigator/AronDataNavigator.h>
#include <RobotAPI/libraries/aron/aroncore/navigators/typenavigator/AronListTypeNavigator.h>

namespace armarx
{
    namespace aron
    {
        namespace datanavigator
        {
            class AronListDataNavigator;
            typedef std::shared_ptr<AronListDataNavigator> AronListDataNavigatorPtr;

            class AronListDataNavigator :
                virtual public AronContainerDataNavigator
            {
            public:
                using PointerType = AronListDataNavigatorPtr;

            public:
                // constructors
                AronListDataNavigator(const AronPath& path = AronPath());
                AronListDataNavigator(const data::AronListPtr&, const AronPath& path = AronPath());
                AronListDataNavigator(const data::AronDataList&, const AronPath& path = AronPath());
                AronListDataNavigator(const std::vector<AronDataNavigatorPtr>&, const AronPath& path = AronPath());

                // operators
                bool operator==(const AronListDataNavigator&) const;
                virtual bool equalsDataNavigator(const AronDataNavigatorPtr&) const override;
                bool equalsDeep(const AronListDataNavigatorPtr&) const;

                // static methods
                static AronListDataNavigatorPtr DynamicCast(const AronDataNavigatorPtr& n);
                static AronListDataNavigator DynamicCast(AronDataNavigator& n);
                static AronListDataNavigatorPtr DynamicCastAndCheck(const AronDataNavigatorPtr& n);

                static AronListDataNavigatorPtr FromAronListPtr(const data::AronListPtr& aron);
                static data::AronListPtr ToAronListPtr(const AronListDataNavigatorPtr& navigator);

                // public member functions
                data::AronListPtr toAronListPtr() const;

                void addElement(const AronDataNavigatorPtr&);
                AronDataNavigatorPtr getElement(unsigned int) const;
                bool hasElement(unsigned int) const;
                std::vector<AronDataNavigatorPtr> getElements() const;

                void clear();

                // virtual implementations
                virtual data::AronDataPtr getResult() const override;
                virtual std::string getName() const override;
                virtual std::vector<AronDataNavigatorPtr> getChildren() const override;
                virtual size_t childrenSize() const override;

                virtual typenavigator::AronTypeNavigatorPtr recalculateType() const override;
                virtual bool fullfillsType(const typenavigator::AronTypeNavigatorPtr&) const override;

                virtual AronDataNavigatorPtr navigateAbsolute(const AronPath& path) const override;

            private:
                std::vector<AronDataNavigatorPtr> childrenNavigators;
                data::AronListPtr aron;
            };
        }
    }
}
