/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "AronPrimitiveDataNavigator.h"

#include <ArmarXCore/core/exceptions/Exception.h>
#include <RobotAPI/libraries/aron/aroncore/navigators/datanavigator/AronDataNavigatorFactory.h>
#include <RobotAPI/libraries/aron/aroncore/navigators/typenavigator/AronPrimitiveTypeNavigator.h>

namespace armarx
{
    namespace aron
    {
        namespace datanavigator
        {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    /* constructors */ \
    Aron##upperType##DataNavigator::Aron##upperType##DataNavigator(const data::Aron##upperType##Ptr& o, const AronPath& path) : \
        AronNavigator<AronDataDescriptor, data::AronData>::AronNavigator(AronDataDescriptor::eAron##upperType, path), \
        AronDataNavigator(AronDataDescriptor::eAron##upperType, path), \
        aron(o) \
    { \
    } \
    \
    Aron##upperType##DataNavigator::Aron##upperType##DataNavigator(const AronPath& path) : \
        AronNavigator<AronDataDescriptor, data::AronData>::AronNavigator(AronDataDescriptor::eAron##upperType, path), \
        AronDataNavigator(AronDataDescriptor::eAron##upperType, path), \
        aron(new data::Aron##upperType()) \
    { \
    } \
    \
    Aron##upperType##DataNavigator::Aron##upperType##DataNavigator(const lowerType& d, const AronPath& path) : \
        Aron##upperType##DataNavigator(data::Aron##upperType##Ptr(new data::Aron##upperType(d)), path) \
    { \
    } \
    \
    /* operators */ \
    bool Aron##upperType##DataNavigator::operator==(const Aron##upperType##DataNavigator& other) const \
    { \
        const auto& otherAron = other.toAron##upperType##Ptr(); \
        if(aron->value != otherAron->value) \
            return false; \
        return true; \
    } \
    \
    bool Aron##upperType##DataNavigator::equalsDataNavigator(const AronDataNavigatorPtr& other) const \
    { \
        if(other == nullptr) \
        { \
            return false; \
        } \
        Aron##upperType##DataNavigatorPtr casted = DynamicCast(other); \
        return equalsDeep(casted); \
    } \
    \
    bool Aron##upperType##DataNavigator::equalsDeep(const Aron##upperType##DataNavigatorPtr& other) const \
    { \
        /* only for consistency. There is no "deep" comparison. */ \
        return *this == *other; \
    } \
    \
    /* static methods */ \
    Aron##upperType##DataNavigatorPtr Aron##upperType##DataNavigator::DynamicCast(const AronDataNavigatorPtr& n) \
    { \
        Aron##upperType##DataNavigatorPtr casted = std::dynamic_pointer_cast<Aron##upperType##DataNavigator>(n); \
        return casted; \
    } \
    \
    Aron##upperType##DataNavigator Aron##upperType##DataNavigator::DynamicCast(AronDataNavigator& n) \
    { \
        return dynamic_cast<Aron##upperType##DataNavigator&>(n); \
    } \
    \
    Aron##upperType##DataNavigatorPtr Aron##upperType##DataNavigator::DynamicCastAndCheck(const AronDataNavigatorPtr& n) \
    { \
        CheckDataNavigatorPtrForNull("Aron"+std::string(#upperType)+"DataNavigator", "DynamicCastAndCheck[Before]", n); \
        Aron##upperType##DataNavigatorPtr casted = Aron##upperType##DataNavigator::DynamicCast(n); \
        CheckDataNavigatorPtrForNull("Aron"+std::string(#upperType)+"DataNavigator", "DynamicCastAndCheck[After]", n->getPath(), casted); \
        return casted; \
    } \
    \
    Aron##upperType##DataNavigatorPtr Aron##upperType##DataNavigator::FromAron##upperType##Ptr(const data::Aron##upperType##Ptr& aron) \
    { \
        return std::make_shared<Aron##upperType##DataNavigator>(aron); \
    } \
    \
    data::Aron##upperType##Ptr Aron##upperType##DataNavigator::ToAron##upperType##Ptr(const Aron##upperType##DataNavigatorPtr& navigator) \
    { \
        return navigator ? navigator->toAron##upperType##Ptr() : nullptr; \
    } \
    \
    /* public member functions */ \
    void Aron##upperType##DataNavigator::setValue(const lowerType& x) \
    { \
        aron->value = x; \
    } \
    \
    lowerType Aron##upperType##DataNavigator::getValue() const \
    { \
        return aron->value; \
    } \
    \
    data::Aron##upperType##Ptr Aron##upperType##DataNavigator::toAron##upperType##Ptr() const \
    { \
        return aron; \
    } \
    \
    /* virtual implementations */ \
    data::AronDataPtr Aron##upperType##DataNavigator::getResult() const \
    { \
        return aron; \
    } \
    \
    std::string Aron##upperType##DataNavigator::getName() const \
    { \
        return "Aron" + std::string(#upperType); \
    } \
    \
    bool Aron##upperType##DataNavigator::fullfillsType(const typenavigator::AronTypeNavigatorPtr& type) const \
    { \
        return AronResolver::TypesAreCorresponding(type->getDescriptor(), getDescriptor()); \
    } \
    \
    typenavigator::AronTypeNavigatorPtr Aron##upperType##DataNavigator::recalculateType() const \
    { \
        return typenavigator::Aron##upperType##TypeNavigatorPtr(new typenavigator::Aron##upperType##TypeNavigator(getPath())); \
    } \
    \
    std::vector<AronDataNavigatorPtr> Aron##upperType##DataNavigator::getChildren() const \
    { \
        return {}; \
    } \
    \
    size_t Aron##upperType##DataNavigator::childrenSize() const \
    { \
        return 0; \
    } \

            HANDLE_PRIMITIVE_TYPES
#undef RUN_ARON_MACRO
        }
    }
}
