/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>

// ArmarX
#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/aroncore/navigators/datanavigator/AronDataNavigator.h>

namespace armarx
{
    namespace aron
    {
        namespace datanavigator
        {

#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    class Aron##upperType##DataNavigator; \
    typedef std::shared_ptr<Aron##upperType##DataNavigator> Aron##upperType##DataNavigatorPtr;

            HANDLE_PRIMITIVE_TYPES
#undef RUN_ARON_MACRO


#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    class Aron##upperType##DataNavigator : \
        virtual public AronPrimitiveDataNavigator \
    { \
    public: \
        using PointerType = Aron##upperType##DataNavigatorPtr; \
        \
        /* constructors */ \
        Aron##upperType##DataNavigator(const AronPath& = AronPath()); \
        Aron##upperType##DataNavigator(const data::Aron##upperType##Ptr&, const AronPath& = AronPath()); \
        Aron##upperType##DataNavigator(const lowerType&, const AronPath& = AronPath()); \
        \
        /* operators */ \
        bool operator==(const Aron##upperType##DataNavigator&) const; \
        virtual bool equalsDataNavigator(const AronDataNavigatorPtr&) const override; \
        bool equalsDeep(const Aron##upperType##DataNavigatorPtr&) const; \
        \
        /* static methods */ \
        static Aron##upperType##DataNavigatorPtr DynamicCast(const AronDataNavigatorPtr& n); \
        static Aron##upperType##DataNavigator DynamicCast(AronDataNavigator& n); \
        static Aron##upperType##DataNavigatorPtr DynamicCastAndCheck(const AronDataNavigatorPtr& n); \
        \
        static Aron##upperType##DataNavigatorPtr FromAron##upperType##Ptr(const data::Aron##upperType##Ptr& aron); \
        static data::Aron##upperType##Ptr ToAron##upperType##Ptr(const Aron##upperType##DataNavigatorPtr& navigator); \
        \
        /* public member functions */ \
        void setValue(const lowerType& x); \
        lowerType getValue() const; \
        \
        data::Aron##upperType##Ptr toAron##upperType##Ptr() const; \
        \
        /* virtual implementations */ \
        virtual data::AronDataPtr getResult() const override; \
        virtual std::string getName() const override; \
        virtual std::vector<AronDataNavigatorPtr> getChildren() const override; \
        virtual size_t childrenSize() const override; \
        \
        virtual typenavigator::AronTypeNavigatorPtr recalculateType() const override; \
        virtual bool fullfillsType(const typenavigator::AronTypeNavigatorPtr&) const override; \
        \
    private: \
        /* members */ \
        data::Aron##upperType##Ptr aron; \
    };

            HANDLE_PRIMITIVE_TYPES
#undef RUN_ARON_MACRO
        }
    }
}
