/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "AronListTypeNavigator.h"

namespace armarx
{
    namespace aron
    {
        namespace typenavigator
        {
            // constructors
            AronListTypeNavigator::AronListTypeNavigator(const AronPath& path) :
                navigator::AronNavigator<AronTypeDescriptor, type::AronType>::AronNavigator(AronTypeDescriptor::eAronListType, path),
                AronTypeNavigator(AronTypeDescriptor::eAronListType, path),
                type(new type::AronListType())
            {
            }

            AronListTypeNavigator::AronListTypeNavigator(const type::AronListTypePtr& o, const AronPath& path) :
                navigator::AronNavigator<AronTypeDescriptor, type::AronType>::AronNavigator(AronTypeDescriptor::eAronListType, path),
                AronTypeNavigator(AronTypeDescriptor::eAronListType, path),
                type(o)
            {
                CheckAronPtrForNull("AronListTypeNavigator", "AronListTypeNavigator", getPath(), o);

                acceptedTypeNavigator = FromAronType(o->acceptedType);
            }

            // static methods
            AronListTypeNavigatorPtr AronListTypeNavigator::DynamicCast(const AronTypeNavigatorPtr& n)
            {
                return std::dynamic_pointer_cast<AronListTypeNavigator>(n);
            }

            AronListTypeNavigatorPtr AronListTypeNavigator::DynamicCastAndCheck(const AronTypeNavigatorPtr& n)
            {
                CheckTypeNavigatorPtrForNull("AronListTypeNavigator", "DynamicCast[Before]", n);
                AronListTypeNavigatorPtr casted = AronListTypeNavigator::DynamicCast(n);
                CheckTypeNavigatorPtrForNull("AronListTypeNavigator", "DynamicCast[After]", n->getPath(), casted);
                return casted;
            }

            type::AronListTypePtr AronListTypeNavigator::toAronListTypePtr() const
            {
                CheckAronPtrForNull("AronListTypeNavigator", "getCastedResult", getPath(), type->acceptedType);
                return type;
            }

            // virtual implementations
            type::AronTypePtr AronListTypeNavigator::getResult() const
            {
                return toAronListTypePtr();
            }

            std::string AronListTypeNavigator::getName() const
            {
                return "AronListType<" + acceptedTypeNavigator->getName() + ">";
            }

            void AronListTypeNavigator::setAcceptedType(const AronTypeNavigatorPtr& a)
            {
                CheckTypeNavigatorPtrForNull("AronListTypeNavigator", "setAcceptedType", a);
                type->acceptedType = a->getResult();
                acceptedTypeNavigator = a;
            }
        }
    }
}
