/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "AronObjectTypeNavigator.h"


namespace armarx
{
    namespace aron
    {
        namespace typenavigator
        {

            // constructors
            AronObjectTypeNavigator::AronObjectTypeNavigator(const AronPath& path) :
                navigator::AronNavigator<AronTypeDescriptor, type::AronType>::AronNavigator(AronTypeDescriptor::eAronObjectType, path),
                AronTypeNavigator(AronTypeDescriptor::eAronObjectType, path),
                type((new type::AronObjectType()))
            {
            }

            AronObjectTypeNavigator::AronObjectTypeNavigator(const type::AronObjectTypePtr& o, const AronPath& path) :
                navigator::AronNavigator<AronTypeDescriptor, type::AronType>::AronNavigator(AronTypeDescriptor::eAronObjectType, path),
                AronTypeNavigator(AronTypeDescriptor::eAronObjectType, path),
                type(o)
            {
                CheckAronPtrForNull("AronObjectTypeNavigator", "AronObjectTypeNavigator", getPath(), o);

                for(const auto& [key, t] : o->elementTypes)
                {
                    acceptedTypeNavigators[key] = FromAronType(t);
                }
            }

            // static methods
            AronObjectTypeNavigatorPtr AronObjectTypeNavigator::DynamicCast(const AronTypeNavigatorPtr& n)
            {
                return std::dynamic_pointer_cast<AronObjectTypeNavigator>(n);
            }

            AronObjectTypeNavigatorPtr AronObjectTypeNavigator::DynamicCastAndCheck(const AronTypeNavigatorPtr& n)
            {
                CheckTypeNavigatorPtrForNull("AronObjectTypeNavigator", "DynamicCast[Before]", n);
                AronObjectTypeNavigatorPtr casted = AronObjectTypeNavigator::DynamicCast(n);
                CheckTypeNavigatorPtrForNull("AronObjectTypeNavigator", "DynamicCast[After]", n->getPath(), casted);
                return casted;
            }

            // public member functions
            void AronObjectTypeNavigator::addAcceptedType(const std::string& k, const AronTypeNavigatorPtr& v)
            {
                if (k.empty())
                {
                    throw exception::AronExceptionWithPathInfo("AronObjectTypeNavigator", "addAcceptedType", "Cannot set an element with an empty key.", getPath());
                }
                CheckTypeNavigatorPtrForNull("AronObjectTypeNavigator", "addAcceptedType", v);
                type->elementTypes[k] = v->getResult();
                acceptedTypeNavigators[k] = v;
            }

            void AronObjectTypeNavigator::setObjectName(const std::string& n)
            {
                //path.setRootIdentifier(n);
                type->objectName = n;
            }

            void AronObjectTypeNavigator::setExtends(const AronObjectTypeNavigatorPtr& p)
            {
                CheckTypeNavigatorPtrForNull("AronObjectTypeNavigator", "setExtends", p);
                type::AronObjectTypePtr ex = type::AronObjectTypePtr::dynamicCast(p->getResult());
                CheckAronPtrForNull("AronObjectTypeNavigator", "setExtends", getPath(), ex);
                extends = p;
            }

            std::string AronObjectTypeNavigator::getObjectName() const
            {
                return type->objectName;
            }

            AronObjectTypeNavigatorPtr AronObjectTypeNavigator::getExtends() const
            {
                return extends;
            }

            type::AronObjectTypePtr AronObjectTypeNavigator::toAronObjectTypePtr() const
            {
                if (type->objectName.empty())
                {
                    throw exception::AronExceptionWithPathInfo("AronObjectTypeNavigator", "getResult", "The object name is empty.", getPath());
                }
                //if(acceptedTypeNavigators.empty())
                //{
                //    throw exception::AronExceptionWithPathInfo("AronObjectTypeNavigator", "getResult", "No accepted types set", getPath());
                //}
                return type;
            }

            // virtual implementations
            type::AronTypePtr AronObjectTypeNavigator::getResult() const
            {
                return toAronObjectTypePtr();
            }

            std::string AronObjectTypeNavigator::getName() const
            {
                return "AronObjectType<" + type->objectName + ">";
            }
        }
    }
}

