/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "AronTupleTypeNavigator.h"


namespace armarx
{
    namespace aron
    {
        namespace typenavigator
        {
            // constructors
            AronTupleTypeNavigator::AronTupleTypeNavigator(const AronPath& path) :
                navigator::AronNavigator<AronTypeDescriptor, type::AronType>::AronNavigator(AronTypeDescriptor::eAronTupleType, path),
                AronTypeNavigator(AronTypeDescriptor::eAronTupleType, path),
                type(new type::AronTupleType())
            {
            }

            AronTupleTypeNavigator::AronTupleTypeNavigator(const type::AronTupleTypePtr& o, const AronPath& path) :
                navigator::AronNavigator<AronTypeDescriptor, type::AronType>::AronNavigator(AronTypeDescriptor::eAronTupleType, path),
                AronTypeNavigator(AronTypeDescriptor::eAronTupleType, path),
                type(o)
            {
                CheckAronPtrForNull("AronTupleTypeNavigator", "AronTupleTypeNavigator", getPath(), o);

                for (const auto& t : type->elementTypes)
                {
                    acceptedTypeNavigators.push_back(FromAronType(t));
                }
            }

            type::AronTupleTypePtr AronTupleTypeNavigator::toAronTupleTypePtr() const
            {
                if (acceptedTypeNavigators.empty())
                {
                    throw exception::AronExceptionWithPathInfo("AronTupleTypeNavigator", "getCastedResult", "No accepted types set", getPath());
                }
                return type;
            }

            // static methods
            AronTupleTypeNavigatorPtr AronTupleTypeNavigator::DynamicCast(const AronTypeNavigatorPtr& n)
            {
                return std::dynamic_pointer_cast<AronTupleTypeNavigator>(n);
            }

            AronTupleTypeNavigatorPtr AronTupleTypeNavigator::DynamicCastAndCheck(const AronTypeNavigatorPtr& n)
            {
                CheckTypeNavigatorPtrForNull("AronTupleTypeNavigator", "DynamicCast[Before]", n);
                AronTupleTypeNavigatorPtr casted = AronTupleTypeNavigator::DynamicCast(n);
                CheckTypeNavigatorPtrForNull("AronTupleTypeNavigator", "DynamicCast[After]", n->getPath(), casted);
                return casted;
            }

            // public member functions
            void AronTupleTypeNavigator::addAcceptedType(const AronTypeNavigatorPtr& v)
            {
                CheckTypeNavigatorPtrForNull("AronTupleTypeNavigator", "addAcceptedType", getPath(), v);
                type->elementTypes.push_back(v->getResult());
                acceptedTypeNavigators.push_back(v);
            }

            // virtual implementations
            type::AronTypePtr AronTupleTypeNavigator::getResult() const
            {
                return toAronTupleTypePtr();
            }

            std::string AronTupleTypeNavigator::getName() const
            {
                std::vector<std::string> names;
                for (const auto n : acceptedTypeNavigators)
                {
                    names.push_back(n->getName());
                }
                return "AronTupleType<" + simox::alg::to_string(names, ", ") + ">";
            }
        }
    }
}

