/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// STD/STL

// Header
#include "AronTypeNavigator.h"

// ArmarX
#include <RobotAPI/libraries/aron/aroncore/navigators/typenavigator/AronTypeNavigatorFactory.h>
#include <RobotAPI/libraries/aron/aroncore/navigators/typenavigator/AronObjectTypeNavigator.h>

namespace armarx
{
    namespace aron
    {
        namespace typenavigator
        {
            /* AronTypeNavigator */
            // static data members
            const AronTypeNavigatorFactoryPtr AronTypeNavigator::FACTORY = AronTypeNavigatorFactoryPtr(new AronTypeNavigatorFactory());

            // constructors
            AronTypeNavigator::AronTypeNavigator(const AronTypeDescriptor& d, const AronPath& path) :
                navigator::AronNavigator<AronTypeDescriptor, type::AronType>::AronNavigator(d, path)
            {
            }

            AronTypeNavigatorPtr AronTypeNavigator::FromAronType(const type::AronTypePtr& a, const AronPath& path)
            {
                return FACTORY->create(a, path);
            }

            void AronTypeNavigator::CheckTypeNavigatorPtrForNull(const std::string& c, const std::string& m, const AronTypeNavigatorPtr& data)
            {
                if (data.get() == nullptr)
                {
                    throw exception::AronException(c, m, "Could not cast an AronTypeNavigatorPtr. The Ptr was NULL.");
                }
            }

            void AronTypeNavigator::CheckTypeNavigatorPtrForNull(const std::string& c, const std::string& m, const AronPath& p, const AronTypeNavigatorPtr& data)
            {
                if (data.get() == nullptr)
                {
                    throw exception::AronExceptionWithPathInfo(c, m, "Could not cast an AronTypeNavigatorPtr. The Ptr was NULL.", p);
                }
            }

            /* AronSingleAcceptedTypeHavingTypeNavigator */
            AronTypeNavigatorPtr AronSingleAcceptedTypeHavingTypeNavigator::getAcceptedType() const
            {
                return acceptedTypeNavigator;
            }

            size_t AronSingleAcceptedTypeHavingTypeNavigator::childrenSize() const
            {
                return 1;
            }

            /* AronMultipleListAcceptedTypeHavingTypeNavigator */
            std::vector<AronTypeNavigatorPtr> AronMultipleListAcceptedTypeHavingTypeNavigator::getAcceptedTypes() const
            {
                return acceptedTypeNavigators;
            }

            AronTypeNavigatorPtr AronMultipleListAcceptedTypeHavingTypeNavigator::getAcceptedType(unsigned int i) const
            {
                if (i >= acceptedTypeNavigators.size())
                {
                    return nullptr;
                }
                return acceptedTypeNavigators[i];
            }

            size_t AronMultipleListAcceptedTypeHavingTypeNavigator::childrenSize() const
            {
                return acceptedTypeNavigators.size();
            }

            /* AronMultipleDictAcceptedTypeHavingTypeNavigator */
            std::map<std::string, AronTypeNavigatorPtr> AronMultipleDictAcceptedTypeHavingTypeNavigator::getAcceptedTypes() const
            {
                return acceptedTypeNavigators;
            }

            AronTypeNavigatorPtr AronMultipleDictAcceptedTypeHavingTypeNavigator::getAcceptedType(const std::string& s) const
            {
                if (acceptedTypeNavigators.find(s) == acceptedTypeNavigators.end())
                {
                    return nullptr;
                }
                return acceptedTypeNavigators.at(s);
            }

            size_t AronMultipleDictAcceptedTypeHavingTypeNavigator::childrenSize() const
            {
                return acceptedTypeNavigators.size();
            }

            std::vector<std::string> AronMultipleDictAcceptedTypeHavingTypeNavigator::getAllKeys() const
            {
                std::vector<std::string> ret;
                for (const auto& [key, _] : acceptedTypeNavigators)
                {
                    ret.push_back(key);
                }
                return ret;
            }

            /* AronComplexTypeNavigator */
            size_t AronComplexTypeNavigator::childrenSize() const
            {
                return 0;
            }

            /* AronPrimitiveTypeNavigator */
            size_t AronPrimitiveTypeNavigator::childrenSize() const
            {
                return 0;
            }

        }
    }
}


