/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// STD/STL

// Header
#include "AronTypeNavigatorFactory.h"

namespace armarx
{
    namespace aron
    {
        namespace typenavigator
        {

            // Map types to factories
            const std::map<AronTypeDescriptor, AronTypeNavigatorFactoryPtr> AronTypeNavigatorFactory::FACTORIES =
            {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    {AronResolver::GetDescriptorForAronTypeId(typeid(type::Aron##upperType##Type)), AronTypeNavigatorFactoryPtr(new Aron##upperType##TypeNavigatorFactory())}, \

                HANDLE_CONTAINER_TYPES
                HANDLE_COMPLEX_TYPES
                HANDLE_PRIMITIVE_TYPES
#undef RUN_ARON_MACRO
            };

            // Access method
            AronTypeNavigatorPtr AronTypeNavigatorFactory::create(const type::AronTypePtr& aron, const AronPath& path) const
            {
                CheckIfInputIsNull("AronTypeNavigatorFactory", "create", path, aron);

                auto typeDescriptor = AronResolver::GetDescriptorForAronType(aron);
                auto factory_iterator = FACTORIES.find(typeDescriptor);
                if (factory_iterator == FACTORIES.end())
                {
                    throw exception::AronExceptionWithPathInfo("AronTypeNavigatorFactory", "create", "Cannot find the desired factory for input: " + AronTypeDescriptorToString.at(typeDescriptor) + ". Cannot create navigator", path);
                }
                return factory_iterator->second->createSpecific(aron, path);
            }

            AronTypeNavigatorPtr AronTypeNavigatorFactory::createSpecific(const type::AronTypePtr&, const AronPath& p) const
            {
                throw exception::AronExceptionWithPathInfo("AronTypeNavigatorFactory", "createSpecific", "Called disallowed method of an AronTypeNavigatorFactory. Use child class instead!", p);
            }

            // Factories
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    AronTypeNavigatorPtr Aron##upperType##TypeNavigatorFactory::createSpecific(const type::AronTypePtr& aron, const AronPath& path) const \
    { \
        CheckIfInputIsNull("Aron"+std::string(#upperType)+"TypeNavigatorFactory", "create", path, aron); \
        type::Aron##upperType##TypePtr aronCasted = type::Aron##upperType##TypePtr::dynamicCast(aron); \
        CheckIfInputIsNull("Aron"+std::string(#upperType)+"TypeNavigatorFactory", "create", path, aronCasted); \
        return AronTypeNavigatorPtr(new Aron##upperType##TypeNavigator(aronCasted, path)); \
    }

            HANDLE_CONTAINER_TYPES
            HANDLE_COMPLEX_TYPES
            HANDLE_PRIMITIVE_TYPES
#undef RUN_ARON_MACRO

        }
    }
}
